@@ -12,13 +12,21 @@
 
 #include "openmpt/all/BuildSettings.hpp"
 
-#if MPT_CXX_AT_LEAST(20) && !defined(MPT_LIBCXX_QUIRK_NO_CHRONO_DATE)
+#if defined(MODPLUG_TRACKER)
+#include "../misc/mptOS.h"
+#include <stdexcept>
+#endif
+
+#if MPT_CXX_AT_LEAST(20) && !defined(MPT_LIBCXX_QUIRK_NO_CHRONO) && !defined(MPT_LIBCXX_QUIRK_NO_CHRONO_DATE)
 #include <chrono>
 #include <exception>
+#elif MPT_CXX_AT_LEAST(17) && !defined(MPT_LIBCXX_QUIRK_NO_CHRONO) && defined(MODPLUG_TRACKER)
+#include <chrono>
 #endif
-#include <string>
 
+#if MPT_CXX_BEFORE(20) || defined(MPT_LIBCXX_QUIRK_NO_CHRONO) || defined(MPT_LIBCXX_QUIRK_NO_CHRONO_DATE)
 #include <ctime>
+#endif
 
 #if MPT_WINNT_AT_LEAST(MPT_WIN_8)
 #define MPT_FALLBACK_TIMEZONE_WINDOWS_HISTORIC
@@ -37,11 +45,35 @@ OPENMPT_NAMESPACE_BEGIN
 
 
 
+#if defined(MODPLUG_TRACKER) && !defined(MPT_LIBCXX_QUIRK_NO_CHRONO)
+
+namespace mpt {
+namespace chrono {
+#if MPT_CXX_AT_LEAST(20)
+using days = std::chrono::days;
+using weeks = std::chrono::weeks;
+using years = std::chrono::years;
+using months = std::chrono::months;
+#else
+using days = std::chrono::duration<int, std::ratio_multiply<std::ratio<24>, std::chrono::hours::period>>;
+using weeks = std::chrono::duration<int, std::ratio_multiply<std::ratio<7>, mpt::chrono::days::period>>;
+using years = std::chrono::duration<int, std::ratio_multiply<std::ratio<146097, 400>, mpt::chrono::days::period>>;
+using months = std::chrono::duration<int, std::ratio_divide<mpt::chrono::years::period, std::ratio<12>>>;
+#endif
+}
+}
+
+#endif // !MPT_LIBCXX_QUIRK_NO_CHRONO
+
+
+
 namespace mpt
 {
 namespace Date
 {
 
+
+
 #if defined(MODPLUG_TRACKER)
 
 #if MPT_OS_WINDOWS
@@ -60,6 +92,8 @@ mpt::ustring ToUString(uint64 time100ns); // i.e. 2015-01-15 18:32:01.718
 
 #endif // MODPLUG_TRACKER
 
+
+
 enum class LogicalTimezone
 {
 	Unspecified,
@@ -146,11 +180,15 @@ struct Unix
 	}
 };
 
+#if MPT_CXX_BEFORE(20) || defined(MPT_LIBCXX_QUIRK_NO_CHRONO) || defined(MPT_LIBCXX_QUIRK_NO_CHRONO_DATE)
+
 inline Unix UnixNow()
 {
 	return Unix{static_cast<int64>(std::time(nullptr))};
 }
 
+#endif
+
 inline int64 UnixAsSeconds(Unix tp)
 {
 	return tp.value;
@@ -175,7 +213,7 @@ Local UnixAsLocal(Unix tp);
 
 } // namespace nochrono
 
-#if MPT_CXX_AT_LEAST(20) && !defined(MPT_LIBCXX_QUIRK_NO_CHRONO_DATE)
+#if MPT_CXX_AT_LEAST(20) && !defined(MPT_LIBCXX_QUIRK_NO_CHRONO) && !defined(MPT_LIBCXX_QUIRK_NO_CHRONO_DATE)
 
 using Unix = std::chrono::system_clock::time_point;
 
@@ -238,8 +276,16 @@ inline mpt::Date::UTC UnixAsUTC(Unix tp)
 
 inline mpt::Date::Unix UnixFromLocal(Local local)
 {
+#if !defined(MPT_LIBCXX_QUIRK_CHRONO_DATE_NO_ZONED_TIME)
 	try
 	{
+#if defined(MODPLUG_TRACKER)
+		if(mpt::OS::Windows::IsWine())
+		{
+			// <https://bugs.openmpt.org/view.php?id=1933>
+			throw std::runtime_error("zoned_time is blacklisted on Wine");
+		}
+#endif
 		std::chrono::time_point<std::chrono::local_t, std::chrono::seconds> local_tp =
 			std::chrono::local_days {
 				std::chrono::year{ local.year } /
@@ -255,6 +301,7 @@ inline mpt::Date::Unix UnixFromLocal(Local local)
 		return std::chrono::zoned_time{std::chrono::current_zone(), local_tp}.get_sys_time();
 #endif
 	} catch(const std::exception &)
+#endif
 	{
 		return mpt::Date::UnixFromSeconds(mpt::Date::nochrono::UnixAsSeconds(mpt::Date::nochrono::UnixFromLocal(local)));
 	}
@@ -262,8 +309,16 @@ inline mpt::Date::Unix UnixFromLocal(Local local)
 
 inline mpt::Date::Local UnixAsLocal(Unix tp)
 {
+#if !defined(MPT_LIBCXX_QUIRK_CHRONO_DATE_NO_ZONED_TIME)
 	try
 	{
+#if defined(MODPLUG_TRACKER)
+		if(mpt::OS::Windows::IsWine())
+		{
+			// <https://bugs.openmpt.org/view.php?id=1933>
+			throw std::runtime_error("zoned_time is blacklisted on Wine");
+		}
+#endif
 		std::chrono::zoned_time local_tp{ std::chrono::current_zone(), tp };
 		std::chrono::local_days dp = std::chrono::floor<std::chrono::days>(local_tp.get_local_time());
 		std::chrono::year_month_day ymd{dp};
@@ -277,6 +332,7 @@ inline mpt::Date::Local UnixAsLocal(Unix tp)
 		result.seconds = static_cast<int64>(hms.seconds().count());
 		return result;
 	} catch(const std::exception &)
+#endif
 	{
 		return mpt::Date::nochrono::UnixAsLocal(mpt::Date::nochrono::UnixFromSeconds(mpt::Date::UnixAsSeconds(tp)));
 	}
@@ -317,52 +373,4 @@ mpt::ustring ToShortenedISO8601(Local date); // i.e. 2015-01-15T18:32:01
 
 
 
-#ifdef MODPLUG_TRACKER
-
-namespace Util
-{
-
-#if MPT_OS_WINDOWS
-
-// RAII wrapper around timeBeginPeriod/timeEndPeriod/timeGetTime (on Windows).
-// This clock is monotonic, even across changing its resolution.
-// This is needed to synchronize time in Steinberg APIs (ASIO and VST).
-class MultimediaClock
-{
-private:
-	uint32 m_CurrentPeriod;
-private:
-	void Init();
-	void SetPeriod(uint32 ms);
-	void Cleanup();
-public:
-	MultimediaClock();
-	MultimediaClock(uint32 ms);
-	~MultimediaClock();
-public:
-	// Sets the desired resolution in milliseconds, returns the obtained resolution in milliseconds.
-	// A parameter of 0 causes the resolution to be reset to system defaults.
-	// A return value of 0 means the resolution is unknown, but timestamps will still be valid.
-	uint32 SetResolution(uint32 ms);
-	// Returns obtained resolution in milliseconds.
-	// A return value of 0 means the resolution is unknown, but timestamps will still be valid.
-	uint32 GetResolution() const; 
-	// Returns current instantaneous timestamp in milliseconds.
-	// The epoch (offset) of the timestamps is undefined but constant until the next system reboot.
-	// The resolution is the value returned from GetResolution().
-	uint32 Now() const;
-	// Returns current instantaneous timestamp in nanoseconds.
-	// The epoch (offset) of the timestamps is undefined but constant until the next system reboot.
-	// The resolution is the value returned from GetResolution() in milliseconds.
-	uint64 NowNanoseconds() const;
-};
-
-#endif // MPT_OS_WINDOWS
-
-} // namespace Util
-
-#endif // MODPLUG_TRACKER
-
-
-
 OPENMPT_NAMESPACE_END
