@@ -7,7 +7,11 @@
  * The OpenMPT source code is released under the BSD license. Read LICENSE for more details.
  */
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
+
+#if defined(__MINGW32__) && !defined(__MINGW64__)
+#include <sys/types.h>
+#endif
 
 #include "libopenmpt_internal.h"
 #include "libopenmpt.h"
@@ -19,6 +23,7 @@
 #include <limits>
 #include <new>
 #include <stdexcept>
+#include <string_view>
 
 #include <cmath>
 #include <cstdio>
@@ -29,8 +34,6 @@
 #pragma warning(disable:4702) /* unreachable code */
 #endif
 
-#ifndef NO_LIBOPENMPT_C
-
 namespace openmpt {
 
 static const char * strdup( const char * src ) {
@@ -88,6 +91,8 @@ public:
 static std::string format_exception( const char * const function ) {
 	std::string err;
 	try {
+		// cppcheck false-positive
+		// cppcheck-suppress rethrowNoCurrentException
 		throw;
 	} catch ( const openmpt::exception & e ) {
 		err += function;
@@ -131,6 +136,8 @@ static int error_from_exception( const char * * error_message ) {
 		}
 	}
 	try {
+		// cppcheck false-positive
+		// cppcheck-suppress rethrowNoCurrentException
 		throw;
 
 	} catch ( const std::bad_alloc & e ) {
@@ -735,6 +742,26 @@ int32_t openmpt_module_get_selected_subsong( openmpt_module * mod ) {
 	return -1;
 }
 
+int32_t openmpt_module_get_restart_order( openmpt_module * mod, int32_t subsong ) {
+	try {
+		openmpt::interface::check_soundfile( mod );
+		return mod->impl->get_restart_order( subsong );
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod );
+	}
+	return -1;
+}
+
+int32_t openmpt_module_get_restart_row( openmpt_module * mod, int32_t subsong ) {
+	try {
+		openmpt::interface::check_soundfile( mod );
+		return mod->impl->get_restart_row( subsong );
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod );
+	}
+	return -1;
+}
+
 int openmpt_module_set_repeat_count( openmpt_module * mod, int32_t repeat_count ) {
 	try {
 		openmpt::interface::check_soundfile( mod );
@@ -765,6 +792,16 @@ double openmpt_module_get_duration_seconds( openmpt_module * mod ) {
 	return 0.0;
 }
 
+double openmpt_module_get_time_at_position( openmpt_module * mod, int32_t order, int32_t row ) {
+	try {
+		openmpt::interface::check_soundfile( mod );
+		return mod->impl->get_time_at_position( order, row );
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod );
+	}
+	return -1.0;
+}
+
 double openmpt_module_set_position_seconds( openmpt_module * mod, double seconds ) {
 	try {
 		openmpt::interface::check_soundfile( mod );
@@ -965,6 +1002,15 @@ int32_t openmpt_module_get_current_tempo( openmpt_module * mod ) {
 	}
 	return 0;
 }
+double openmpt_module_get_current_tempo2( openmpt_module * mod ) {
+	try {
+		openmpt::interface::check_soundfile( mod );
+		return mod->impl->get_current_tempo2();
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod );
+	}
+	return 0;
+}
 int32_t openmpt_module_get_current_order( openmpt_module * mod ) {
 	try {
 		openmpt::interface::check_soundfile( mod );
@@ -1210,6 +1256,44 @@ int32_t openmpt_module_get_order_pattern( openmpt_module * mod, int32_t order )
 	return 0;
 }
 
+int openmpt_module_is_order_skip_entry( openmpt_module * mod, int32_t order ) {
+	try {
+		openmpt::interface::check_soundfile( mod );
+		return mod->impl->is_order_skip_entry( order ) ? 1 : 0;
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod );
+	}
+	return 0;
+}
+int openmpt_module_is_pattern_skip_item( openmpt_module * mod, int32_t pattern ) {
+	try {
+		openmpt::interface::check_soundfile( mod );
+		return mod->impl->is_pattern_skip_item( pattern ) ? 1 : 0;
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod );
+	}
+	return 0;
+}
+int openmpt_module_is_order_stop_entry( openmpt_module * mod, int32_t order ) {
+	try {
+		openmpt::interface::check_soundfile( mod );
+		return mod->impl->is_order_stop_entry( order ) ? 1 : 0;
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod );
+	}
+	return 0;
+}
+int openmpt_module_is_pattern_stop_item( openmpt_module * mod, int32_t pattern ) {
+	try {
+		openmpt::interface::check_soundfile( mod );
+		return mod->impl->is_pattern_stop_item( pattern ) ? 1 : 0;
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod );
+	}
+	return 0;
+}
+
+
 int32_t openmpt_module_get_pattern_num_rows( openmpt_module * mod, int32_t pattern ) {
 	try {
 		openmpt::interface::check_soundfile( mod );
@@ -1220,6 +1304,25 @@ int32_t openmpt_module_get_pattern_num_rows( openmpt_module * mod, int32_t patte
 	return 0;
 }
 
+int32_t openmpt_module_get_pattern_rows_per_beat( openmpt_module * mod, int32_t pattern ) {
+	try {
+		openmpt::interface::check_soundfile( mod );
+		return mod->impl->get_pattern_rows_per_beat( pattern );
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod );
+	}
+	return 0;
+}
+int32_t openmpt_module_get_pattern_rows_per_measure( openmpt_module * mod, int32_t pattern ) {
+	try {
+		openmpt::interface::check_soundfile( mod );
+		return mod->impl->get_pattern_rows_per_measure( pattern );
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod );
+	}
+	return 0;
+}
+
 uint8_t openmpt_module_get_pattern_row_channel_command( openmpt_module * mod, int32_t pattern, int32_t row, int32_t channel, int command ) {
 	try {
 		openmpt::interface::check_soundfile( mod );
@@ -1723,6 +1826,80 @@ static int stop_note( openmpt_module_ext * mod_ext, int32_t channel ) {
 
 
 
+static int note_off( openmpt_module_ext * mod_ext, int32_t channel ) {
+	try {
+		openmpt::interface::check_soundfile( mod_ext );
+		mod_ext->impl->note_off(channel );
+		return 1;
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod_ext ? &mod_ext->mod : NULL );
+	}
+	return 0;
+}
+static int note_fade( openmpt_module_ext * mod_ext, int32_t channel ) {
+	try {
+		openmpt::interface::check_soundfile( mod_ext );
+		mod_ext->impl->note_fade(channel );
+		return 1;
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod_ext ? &mod_ext->mod : NULL );
+	}
+	return 0;
+}
+static int set_channel_panning( openmpt_module_ext * mod_ext, int32_t channel, double panning ) {
+	try {
+		openmpt::interface::check_soundfile( mod_ext );
+		mod_ext->impl->set_channel_panning( channel, panning );
+		return 1;
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod_ext ? &mod_ext->mod : NULL );
+	}
+	return 0;
+}
+static double get_channel_panning( openmpt_module_ext * mod_ext, int32_t channel ) {
+	try {
+		openmpt::interface::check_soundfile( mod_ext );
+		return mod_ext->impl->get_channel_panning( channel );
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod_ext ? &mod_ext->mod : NULL );
+	}
+	return 0.0;
+}
+static int set_note_finetune( openmpt_module_ext * mod_ext, int32_t channel, double finetune ) {
+	try {
+		openmpt::interface::check_soundfile( mod_ext );
+		mod_ext->impl->set_note_finetune( channel, finetune );
+		return 1;
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod_ext ? &mod_ext->mod : NULL );
+	}
+	return 0;
+}
+static double get_note_finetune( openmpt_module_ext * mod_ext, int32_t channel ) {
+	try {
+		openmpt::interface::check_soundfile( mod_ext );
+		return mod_ext->impl->get_note_finetune( channel );
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod_ext ? &mod_ext->mod : NULL );
+	}
+	return 0.0;
+}
+
+
+
+static int set_current_tempo2( openmpt_module_ext * mod_ext, double tempo ) {
+	try {
+		openmpt::interface::check_soundfile( mod_ext );
+		mod_ext->impl->set_current_tempo2( tempo );
+		return 1;
+	} catch ( ... ) {
+		openmpt::report_exception( __func__, mod_ext ? &mod_ext->mod : NULL );
+	}
+	return 0;
+}
+
+
+
 /* add stuff here */
 
 
@@ -1734,12 +1911,13 @@ int openmpt_module_ext_get_interface( openmpt_module_ext * mod_ext, const char *
 		openmpt::interface::check_pointer( interface );
 		std::memset( interface, 0, interface_size );
 		int result = 0;
-		if ( !std::strcmp( interface_id, "" ) ) {
+		std::string_view interface_id_sv = interface_id;
+		if ( interface_id_sv == "" ) {
 			result = 0;
 
 
 
-		} else if ( !std::strcmp( interface_id, LIBOPENMPT_EXT_C_INTERFACE_PATTERN_VIS ) && ( interface_size == sizeof( openmpt_module_ext_interface_pattern_vis ) ) ) {
+		} else if ( ( interface_id_sv == LIBOPENMPT_EXT_C_INTERFACE_PATTERN_VIS ) && ( interface_size == sizeof( openmpt_module_ext_interface_pattern_vis ) ) ) {
 			openmpt_module_ext_interface_pattern_vis * i = static_cast< openmpt_module_ext_interface_pattern_vis * >( interface );
 			i->get_pattern_row_channel_volume_effect_type = &get_pattern_row_channel_volume_effect_type;
 			i->get_pattern_row_channel_effect_type = &get_pattern_row_channel_effect_type;
@@ -1747,7 +1925,7 @@ int openmpt_module_ext_get_interface( openmpt_module_ext * mod_ext, const char *
 
 
 
-		} else if ( !std::strcmp( interface_id, LIBOPENMPT_EXT_C_INTERFACE_INTERACTIVE ) && ( interface_size == sizeof( openmpt_module_ext_interface_interactive ) ) ) {
+		} else if ( ( interface_id_sv == LIBOPENMPT_EXT_C_INTERFACE_INTERACTIVE ) && ( interface_size == sizeof( openmpt_module_ext_interface_interactive ) ) ) {
 			openmpt_module_ext_interface_interactive * i = static_cast< openmpt_module_ext_interface_interactive * >( interface );
 			i->set_current_speed = &set_current_speed;
 			i->set_current_tempo = &set_current_tempo;
@@ -1769,6 +1947,25 @@ int openmpt_module_ext_get_interface( openmpt_module_ext * mod_ext, const char *
 
 
 
+		} else if ( ( interface_id_sv == LIBOPENMPT_EXT_C_INTERFACE_INTERACTIVE2 ) && ( interface_size == sizeof( openmpt_module_ext_interface_interactive2 ) ) ) {
+			openmpt_module_ext_interface_interactive2 * i = static_cast< openmpt_module_ext_interface_interactive2 * >( interface );
+			i->note_off = &note_off;
+			i->note_fade = &note_fade;
+			i->set_channel_panning = &set_channel_panning;
+			i->get_channel_panning = &get_channel_panning;
+			i->set_note_finetune = &set_note_finetune;
+			i->get_note_finetune = &get_note_finetune;
+			result = 1;
+
+
+
+		} else if ( ( interface_id_sv == LIBOPENMPT_EXT_C_INTERFACE_INTERACTIVE3 ) && ( interface_size == sizeof( openmpt_module_ext_interface_interactive3 ) ) ) {
+			openmpt_module_ext_interface_interactive3 * i = static_cast< openmpt_module_ext_interface_interactive3 * >( interface );
+			i->set_current_tempo2 = &set_current_tempo2;
+			result = 1;
+
+
+
 /* add stuff here */
 
 
@@ -1784,5 +1981,3 @@ int openmpt_module_ext_get_interface( openmpt_module_ext * mod_ext, const char *
 }
 
 } // extern "C"
-
-#endif // NO_LIBOPENMPT_C
