@@ -218,7 +218,14 @@ std::string get_string( const std::string & key ) {
 	} else if ( key == "library_version_prerel" ) {
 		return mpt::format_value_default<std::string>(OPENMPT_API_VERSION_PREREL);
 	} else if ( key == "library_version_is_release" ) {
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunreachable-code"
+#endif // MPT_COMPILER_CLANG
 		return ( std::string(OPENMPT_API_VERSION_PREREL).length() == 0 ) ? "1" : "0";
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic push
+#endif // MPT_COMPILER_CLANG
 	} else if ( key == "library_features" ) {
 		return get_library_features_string();
 	} else if ( key == "core_version" ) {
