@@ -12,13 +12,9 @@
 
 #include "libopenmpt.h"
 
-#include <limits.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
-#ifdef _MSC_VER
-#include <wchar.h> /* off_t */
-#endif
 
 /*! \addtogroup libopenmpt_c
  * @{
@@ -52,48 +48,38 @@ static int openmpt_stream_file_seek_func( void * stream, int64_t offset, int whe
 		return -1;
 	}
 	switch ( whence ) {
-#if defined(SEEK_SET)
 		case OPENMPT_STREAM_SEEK_SET:
 			fwhence = SEEK_SET;
 			break;
-#endif
-#if defined(SEEK_CUR)
 		case OPENMPT_STREAM_SEEK_CUR:
 			fwhence = SEEK_CUR;
 			break;
-#endif
-#if defined(SEEK_END)
 		case OPENMPT_STREAM_SEEK_END:
 			fwhence = SEEK_END;
 			break;
-#endif
 		default:
 			return -1;
 			break;
 	}
-	#if defined(_MSC_VER)
-		return _fseeki64( f, offset, fwhence ) ? -1 : 0;
-	#elif defined(_POSIX_SOURCE) && (_POSIX_SOURCE == 1) 
-		return fseeko( f, offset, fwhence ) ? -1 : 0;
-	#else
-		return fseek( f, offset, fwhence ) ? -1 : 0;
-	#endif
+	if ( (long)offset != offset ) {
+		return -1;
+	}
+	return fseek( f, (long)offset, fwhence ) ? -1 : 0;
 }
 
 static int64_t openmpt_stream_file_tell_func( void * stream ) {
 	FILE * f = 0;
+	long result = 0;
 	int64_t retval = 0;
 	f = (FILE*)stream;
 	if ( !f ) {
 		return -1;
 	}
-	#if defined(_MSC_VER)
-		retval = _ftelli64( f );
-	#elif defined(_POSIX_SOURCE) && (_POSIX_SOURCE == 1) 
-		retval = ftello( f );
-	#else
-		retval = ftell( f );
-	#endif
+	result = ftell( f );
+	if ( (int64_t)result != result ) {
+		return -1;
+	}
+	retval = (int64_t)result;
 	if ( retval < 0 ) {
 		return -1;
 	}
@@ -103,6 +89,9 @@ static int64_t openmpt_stream_file_tell_func( void * stream ) {
 /*! \brief Provide openmpt_stream_callbacks for standard C FILE objects
  *
  * Fills openmpt_stream_callbacks suitable for passing a standard C FILE object as a stream parameter to functions doing file input/output.
+ * Since 0.7.0, it does not try to use platform-specific file seeking any more,
+ * but sticks to standard C fseek/ftell only, which means on platforms where
+ * long is 32bit, there is no 64bit file access possible any more.
  *
  * \remarks The stream argument must be passed as `(void*)(FILE*)file`.
  * \sa \ref libopenmpt_c_fileio
@@ -110,8 +99,37 @@ static int64_t openmpt_stream_file_tell_func( void * stream ) {
  * \sa openmpt_could_open_probability2
  * \sa openmpt_probe_file_header_from_stream
  * \sa openmpt_module_create2
+ * \sa openmpt_stream_get_file_callbacks2()
+ * \sa openmpt_stream_get_file_mingw_callbacks()
+ * \sa openmpt_stream_get_file_msvcrt_callbacks()
+ * \sa openmpt_stream_get_file_posix_callbacks()
+ * \sa openmpt_stream_get_file_posix_lfs64_callbacks()
+ * \deprecated Please use openmpt_stream_get_file_callbacks2().
  */
-static openmpt_stream_callbacks openmpt_stream_get_file_callbacks(void) {
+LIBOPENMPT_DEPRECATED static LIBOPENMPT_C_INLINE openmpt_stream_callbacks openmpt_stream_get_file_callbacks(void) {
+	openmpt_stream_callbacks retval;
+	memset( &retval, 0, sizeof( openmpt_stream_callbacks ) );
+	retval.read = openmpt_stream_file_read_func;
+	retval.seek = openmpt_stream_file_seek_func;
+	retval.tell = openmpt_stream_file_tell_func;
+	return retval;
+}
+
+/*! \brief Provide openmpt_stream_callbacks for standard C FILE objects
+ *
+ * Fills openmpt_stream_callbacks suitable for passing a standard C FILE object as a stream parameter to functions doing file input/output.
+ * This function uses the standard C fseek and ftell, thus platform-specific file size limits apply accordingly.
+ *
+ * \remarks The stream argument must be passed as `(void*)(FILE*)file`.
+ * \remarks The provided callbacks are limited to handling files up to the size representable in the platform's long type.
+ * \sa \ref libopenmpt_c_fileio
+ * \sa openmpt_stream_callbacks
+ * \sa openmpt_could_open_probability2
+ * \sa openmpt_probe_file_header_from_stream
+ * \sa openmpt_module_create2
+ * \since 0.7.0
+ */
+static openmpt_stream_callbacks openmpt_stream_get_file_callbacks2(void) {
 	openmpt_stream_callbacks retval;
 	memset( &retval, 0, sizeof( openmpt_stream_callbacks ) );
 	retval.read = openmpt_stream_file_read_func;
@@ -129,4 +147,3 @@ static openmpt_stream_callbacks openmpt_stream_get_file_callbacks(void) {
  */
 
 #endif /* LIBOPENMPT_STREAM_CALLBACKS_FILE_H */
-
