@@ -11,7 +11,7 @@
 
 #pragma once
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
 
 
 OPENMPT_NAMESPACE_BEGIN
@@ -43,6 +43,13 @@ public:
 };
 
 
+struct BitCrushSettings
+{
+	int m_Bits;
+	BitCrushSettings();
+};
+
+
 class CSurround
 {
 public:
@@ -116,6 +123,19 @@ public:
 };
 
 
+class BitCrush
+{
+public:
+	BitCrushSettings m_Settings;
+public:
+	BitCrush();
+public:
+	void SetSettings(const BitCrushSettings &settings) { m_Settings = settings; }
+	void Initialize(bool bReset, DWORD MixingFreq);
+	void Process(int * MixSoundBuffer, int * MixRearBuffer, int count, uint32 nChannels);
+};
+
+
 #endif // NO_DSP
 
 
