@@ -25,7 +25,8 @@ OPENMPT_NAMESPACE_BEGIN
 class BitReader : private FileReader
 {
 protected:
-	off_t m_bufPos = 0, m_bufSize = 0;
+	pos_type m_bufPos = 0;
+	pos_type m_bufSize = 0;
 	uint32 bitBuf = 0; // Current bit buffer
 	int m_bitNum = 0;  // Currently available number of bits
 	std::byte buffer[mpt::IO::BUFFERSIZE_TINY]{};
@@ -43,12 +44,12 @@ public:
 	BitReader(const FileCursor &other) : FileReader(other) { }
 	BitReader(FileCursor &&other) : FileReader(std::move(other)) { }
 
-	off_t GetLength() const
+	pos_type GetLength() const
 	{
 		return FileReader::GetLength();
 	}
 
-	off_t GetPosition() const
+	pos_type GetPosition() const
 	{
 		return FileReader::GetPosition() - m_bufSize + m_bufPos;
 	}
