@@ -37,8 +37,8 @@ MPT_BINARY_STRUCT(DIGIFileHeader, 610)
 
 static void ReadDIGIPatternEntry(FileReader &file, ModCommand &m)
 {
-	CSoundFile::ReadMODPatternEntry(file, m);
-	CSoundFile::ConvertModCommand(m);
+	const auto [command, param] = CSoundFile::ReadMODPatternEntry(file, m);
+	CSoundFile::ConvertModCommand(m, command, param);
 	if(m.command == CMD_MODCMDEX)
 	{
 		switch(m.param & 0xF0)
@@ -119,15 +119,12 @@ bool CSoundFile::ReadDIGI(FileReader &file, ModLoadingFlags loadFlags)
 	}
 
 	// Globals
-	InitializeGlobals(MOD_TYPE_DIGI);
-	InitializeChannels();
-
-	m_nChannels = fileHeader.numChannels;
+	InitializeGlobals(MOD_TYPE_DIGI, fileHeader.numChannels);
 	m_nSamples = 31;
-	m_nSamplePreAmp = 256 / m_nChannels;
+	m_nSamplePreAmp = 256 / GetNumChannels();
 
-	m_modFormat.formatName = U_("DigiBooster");
-	m_modFormat.type = U_("digi");
+	m_modFormat.formatName = UL_("DigiBooster");
+	m_modFormat.type = UL_("digi");
 	m_modFormat.madeWithTracker = MPT_UFORMAT("Digi Booster {}.{}")(fileHeader.versionInt >> 4, fileHeader.versionInt & 0x0F);
 	m_modFormat.charset = mpt::Charset::Amiga_no_C1;
 
@@ -185,15 +182,13 @@ bool CSoundFile::ReadDIGI(FileReader &file, ModLoadingFlags loadFlags)
 			// Compressed patterns are stored in row-major order...
 			for(ROWINDEX row = 0; row < 64; row++)
 			{
-				PatternRow patRow = Patterns[pat].GetRow(row);
+				
 				uint8 bit = 0x80;
-				for(CHANNELINDEX chn = 0; chn < GetNumChannels(); chn++, bit >>= 1)
+				for(ModCommand &m : Patterns[pat].GetRow(row))
 				{
 					if(eventMask[row] & bit)
-					{
-						ModCommand &m = patRow[chn];
 						ReadDIGIPatternEntry(patternChunk, m);
-					}
+					bit >>= 1;
 				}
 			}
 		} else
