@@ -34,7 +34,7 @@ struct GDMFileHeader
 	uint16le trackerID;         // Composing Tracker ID code (00 = 2GDM)
 	uint8le  trackerMajorVer;   // Tracker's major version
 	uint8le  trackerMinorVer;   // Tracker's minor version
-	uint8le  panMap[32];        // 0-Left to 15-Right, 255-N/U
+	uint8le  panMap[32];        // 0-Left to 15-Right, 16=Surround, 255-N/U
 	uint8le  masterVol;         // Range: 0...64
 	uint8le  tempo;             // Initial music tempo (6)
 	uint8le  bpm;               // Initial music BPM (125)
@@ -57,6 +57,11 @@ struct GDMFileHeader
 	uint16le scrollyScriptLength;
 	uint32le textGraphicOffset;    // Offset of text graphic (huh?)
 	uint16le textGraphicLength;
+
+	uint8 GetNumChannels() const
+	{
+		return static_cast<uint8>(std::distance(std::begin(panMap), std::find(std::begin(panMap), std::end(panMap), uint8_max)));
+	}
 };
 
 MPT_BINARY_STRUCT(GDMFileHeader, 157)
@@ -120,7 +125,8 @@ static bool ValidateHeader(const GDMFileHeader &fileHeader)
 		|| std::memcmp(fileHeader.magic2, "GMFS", 4)
 		|| fileHeader.formatMajorVer != 1 || fileHeader.formatMinorVer != 0
 		|| fileHeader.originalFormat >= std::size(gdmFormatOrigin)
-		|| fileHeader.originalFormat == 0)
+		|| fileHeader.originalFormat == 0
+		|| !fileHeader.GetNumChannels())
 	{
 		return false;
 	}
@@ -162,11 +168,11 @@ bool CSoundFile::ReadGDM(FileReader &file, ModLoadingFlags loadFlags)
 		return true;
 	}
 
-	InitializeGlobals(gdmFormatOrigin[fileHeader.originalFormat]);
+	InitializeGlobals(gdmFormatOrigin[fileHeader.originalFormat], fileHeader.GetNumChannels());
 	m_SongFlags.set(SONG_IMPORTED);
 
-	m_modFormat.formatName = U_("General Digital Music");
-	m_modFormat.type = U_("gdm");
+	m_modFormat.formatName = UL_("General Digital Music");
+	m_modFormat.type = UL_("gdm");
 	m_modFormat.madeWithTracker = MPT_UFORMAT("BWSB 2GDM {}.{}")(fileHeader.trackerMajorVer, fileHeader.formatMinorVer);
 	m_modFormat.originalType = gdmFormatOriginType[fileHeader.originalFormat];
 	m_modFormat.originalFormatName = gdmFormatOriginFormat[fileHeader.originalFormat];
@@ -185,10 +191,8 @@ bool CSoundFile::ReadGDM(FileReader &file, ModLoadingFlags loadFlags)
 	}
 
 	// Read channel pan map... 0...15 = channel panning, 16 = surround channel, 255 = channel does not exist
-	m_nChannels = 32;
-	for(CHANNELINDEX i = 0; i < 32; i++)
+	for(CHANNELINDEX i = 0; i < GetNumChannels(); i++)
 	{
-		ChnSettings[i].Reset();
 		if(fileHeader.panMap[i] < 16)
 		{
 			ChnSettings[i].nPan = static_cast<uint16>(std::min((fileHeader.panMap[i] * 16) + 8, 256));
@@ -196,20 +200,12 @@ bool CSoundFile::ReadGDM(FileReader &file, ModLoadingFlags loadFlags)
 		{
 			ChnSettings[i].nPan = 128;
 			ChnSettings[i].dwFlags = CHN_SURROUND;
-		} else if(fileHeader.panMap[i] == 0xFF)
-		{
-			m_nChannels = i;
-			break;
 		}
 	}
-	if(m_nChannels < 1)
-	{
-		return false;
-	}
 
 	m_nDefaultGlobalVolume = std::min(fileHeader.masterVol * 4u, 256u);
-	m_nDefaultSpeed = fileHeader.tempo;
-	m_nDefaultTempo.Set(fileHeader.bpm);
+	Order().SetDefaultSpeed(fileHeader.tempo);
+	Order().SetDefaultTempoInt(fileHeader.bpm);
 
 	// Read orders
 	if(file.Seek(fileHeader.orderOffset))
@@ -341,14 +337,14 @@ bool CSoundFile::ReadGDM(FileReader &file, ModLoadingFlags loadFlags)
 
 		for(ROWINDEX row = 0; row < 64; row++)
 		{
-			PatternRow rowBase = Patterns[pat].GetRow(row);
+			auto rowBase = Patterns[pat].GetRow(row);
 
 			uint8 channelByte;
 			// If channel byte is zero, advance to next row.
 			while((channelByte = chunk.ReadUint8()) != rowDone)
 			{
 				CHANNELINDEX channel = channelByte & channelMask;
-				if(channel >= m_nChannels) break; // Better safe than sorry!
+				if(channel >= GetNumChannels()) break; // Better safe than sorry!
 
 				ModCommand &m = rowBase[channel];
 
@@ -360,7 +356,7 @@ bool CSoundFile::ReadGDM(FileReader &file, ModLoadingFlags loadFlags)
 					if(note)
 					{
 						note = (note & 0x7F) - 1;  // High bit = no-retrig flag (notes with portamento have this set)
-						m.note = (note & 0x0F) + 12 * (note >> 4) + 12 + NOTE_MIN;
+						m.note = static_cast<ModCommand::NOTE>((note & 0x0F) + 12 * (note >> 4) + 12 + NOTE_MIN);
 						if(!m.IsAmigaNote())
 						{
 							onlyAmigaNotes = false;
@@ -474,15 +470,16 @@ bool CSoundFile::ReadGDM(FileReader &file, ModLoadingFlags loadFlags)
 								m.command = CMD_NONE;
 							}
 							break;
+
+						default:
+							break;
 						}
 
 						// Move pannings to volume column - should never happen
 						if(m.command == CMD_S3MCMDEX && ((m.param >> 4) == 0x8) && m.volcmd == VOLCMD_NONE)
 						{
-							m.volcmd = VOLCMD_PANNING;
-							m.vol = ((m.param & 0x0F) * 64 + 8) / 15;
-							m.command = oldCmd.command;
-							m.param = oldCmd.param;
+							m.SetVolumeCommand(VOLCMD_PANNING, static_cast<ModCommand::VOL>(((m.param & 0x0F) * 64 + 8) / 15));
+							m.SetEffectCommand(oldCmd);
 						}
 
 						if(!(effByte & effectMore))
