@@ -22,6 +22,8 @@
 #include "../mptrack/Moddoc.h"
 #endif // MODPLUG_TRACKER
 #ifdef MPT_EXTERNAL_SAMPLES
+#include "mpt/io_file/inputfile.hpp"
+#include "mpt/io_file_read/inputfile_filecursor.hpp"
 #include "../common/mptFileIO.h"
 #endif // MPT_EXTERNAL_SAMPLES
 
@@ -84,43 +86,63 @@ struct ITPHeader
 {
 	uint32le magic;
 	uint32le version;
+
+	bool IsValid() const
+	{
+		return magic == MagicBE(".itp")
+			&& version >= 0x00000100 && version <= 0x00000103;
+	}
+
+	uint32 GetHeaderMinimumAdditionalSize() const
+	{
+		return 76 + (version <= 0x102 ? 4 : 0);
+	}
 };
 
 MPT_BINARY_STRUCT(ITPHeader, 8)
 
 
-static bool ValidateHeader(const ITPHeader &hdr)
+struct ITPSongHeader
 {
-	if(hdr.magic != MagicBE(".itp"))
+	enum SongFlags
 	{
-		return false;
-	}
-	if(hdr.version < 0x00000100 || hdr.version > 0x00000103)
+		ITP_EMBEDMIDICFG  = 0x00001,  // Embed macros in file
+		ITP_ITOLDEFFECTS  = 0x00004,  // Old Impulse Tracker effect implementations
+		ITP_ITCOMPATGXX   = 0x00008,  // IT "Compatible Gxx" (IT's flag to behave more like other trackers w/r/t portamento effects)
+		ITP_LINEARSLIDES  = 0x00010,  // Linear slides vs. Amiga slides
+		ITP_EXFILTERRANGE = 0x08000,  // Cutoff Filter has double frequency range (up to ~10Khz)
+		ITP_ITPROJECT     = 0x20000,  // Is a project file
+		ITP_ITPEMBEDIH    = 0x40000,  // Embed instrument headers in project file
+	};
+
+	uint32le flags;  // See SongFlangs
+	uint32le globalVolume;
+	uint32le samplePreAmp;
+	uint32le speed;
+	uint32le tempo;
+	uint32le numChannels;
+	uint32le channelNameLength;
+
+	bool IsValid() const
 	{
-		return false;
+		return (flags & ITP_ITPROJECT) != 0
+			&& speed >= 1
+			&& tempo >= 32
+			&& numChannels >= 1 && numChannels <= MAX_BASECHANNELS;
 	}
-	return true;
-}
+};
 
-
-static uint64 GetHeaderMinimumAdditionalSize(const ITPHeader &hdr)
-{
-	return 76 + (hdr.version <= 0x102 ? 4 : 0);
-}
+MPT_BINARY_STRUCT(ITPSongHeader, 28)
 
 
 CSoundFile::ProbeResult CSoundFile::ProbeFileHeaderITP(MemoryFileReader file, const uint64 *pfilesize)
 {
 	ITPHeader hdr;
 	if(!file.ReadStruct(hdr))
-	{
 		return ProbeWantMoreData;
-	}
-	if(!ValidateHeader(hdr))
-	{
+	if(!hdr.IsValid())
 		return ProbeFailure;
-	}
-	return ProbeAdditionalSize(file, pfilesize, GetHeaderMinimumAdditionalSize(hdr));
+	return ProbeAdditionalSize(file, pfilesize, hdr.GetHeaderMinimumAdditionalSize());
 }
 
 
@@ -133,85 +155,60 @@ bool CSoundFile::ReadITP(FileReader &file, ModLoadingFlags loadFlags)
 	return false;
 #else // !MPT_EXTERNAL_SAMPLES && !MPT_FUZZ_TRACKER
 
-	enum ITPSongFlags
-	{
-		ITP_EMBEDMIDICFG  = 0x00001,  // Embed macros in file
-		ITP_ITOLDEFFECTS  = 0x00004,  // Old Impulse Tracker effect implementations
-		ITP_ITCOMPATGXX   = 0x00008,  // IT "Compatible Gxx" (IT's flag to behave more like other trackers w/r/t portamento effects)
-		ITP_LINEARSLIDES  = 0x00010,  // Linear slides vs. Amiga slides
-		ITP_EXFILTERRANGE = 0x08000,  // Cutoff Filter has double frequency range (up to ~10Khz)
-		ITP_ITPROJECT     = 0x20000,  // Is a project file
-		ITP_ITPEMBEDIH    = 0x40000,  // Embed instrument headers in project file
-	};
-
 	file.Rewind();
 
 	ITPHeader hdr;
 	if(!file.ReadStruct(hdr))
-	{
 		return false;
-	}
-	if(!ValidateHeader(hdr))
-	{
+	if(!hdr.IsValid())
 		return false;
-	}
-	if(!file.CanRead(mpt::saturate_cast<FileReader::off_t>(GetHeaderMinimumAdditionalSize(hdr))))
-	{
+	if(!file.CanRead(hdr.GetHeaderMinimumAdditionalSize()))
 		return false;
-	}
 	if(loadFlags == onlyVerifyHeader)
-	{
 		return true;
-	}
 
 	const uint32 version = hdr.version;
 
-	InitializeGlobals(MOD_TYPE_IT);
-	m_playBehaviour.reset();
-	file.ReadSizedString<uint32le, mpt::String::maybeNullTerminated>(m_songName);
+	std::string songName, songMessage;
+	file.ReadSizedString<uint32le, mpt::String::maybeNullTerminated>(songName);
+	file.ReadSizedString<uint32le, mpt::String::maybeNullTerminated>(songMessage);
 
-	// Song comments
-	m_songMessage.Read(file, file.ReadUint32LE(), SongMessage::leCR);
-
-	// Song global config
-	const uint32 songFlags = file.ReadUint32LE();
-	if(!(songFlags & ITP_ITPROJECT))
-	{
+	ITPSongHeader songHeader;
+	if(!file.ReadStruct(songHeader) || !songHeader.IsValid())
 		return false;
-	}
+
+	InitializeGlobals(MOD_TYPE_IT, static_cast<CHANNELINDEX>(songHeader.numChannels));
+	m_playBehaviour.reset();
+
 	m_SongFlags.set(SONG_IMPORTED);
-	if(songFlags & ITP_ITOLDEFFECTS)
+	if(songHeader.flags & ITPSongHeader::ITP_ITOLDEFFECTS)
 		m_SongFlags.set(SONG_ITOLDEFFECTS);
-	if(songFlags & ITP_ITCOMPATGXX)
+	if(songHeader.flags & ITPSongHeader::ITP_ITCOMPATGXX)
 		m_SongFlags.set(SONG_ITCOMPATGXX);
-	if(songFlags & ITP_LINEARSLIDES)
+	if(songHeader.flags & ITPSongHeader::ITP_LINEARSLIDES)
 		m_SongFlags.set(SONG_LINEARSLIDES);
-	if(songFlags & ITP_EXFILTERRANGE)
+	if(songHeader.flags & ITPSongHeader::ITP_EXFILTERRANGE)
 		m_SongFlags.set(SONG_EXFILTERRANGE);
 
-	m_nDefaultGlobalVolume = file.ReadUint32LE();
-	m_nSamplePreAmp = file.ReadUint32LE();
-	m_nDefaultSpeed = std::max(uint32(1), file.ReadUint32LE());
-	m_nDefaultTempo.Set(std::max(uint32(32), file.ReadUint32LE()));
-	m_nChannels = static_cast<CHANNELINDEX>(file.ReadUint32LE());
-	if(m_nChannels == 0 || m_nChannels > MAX_BASECHANNELS)
-	{
-		return false;
-	}
+	m_nDefaultGlobalVolume = songHeader.globalVolume;
+	m_nSamplePreAmp = songHeader.samplePreAmp;
+	Order().SetDefaultSpeed(songHeader.speed);
+	Order().SetDefaultTempoInt(songHeader.tempo);
 
-	// channel name string length (=MAX_CHANNELNAME)
-	uint32 size = file.ReadUint32LE();
+	m_songName = std::move(songName);
+	m_songMessage.SetRaw(std::move(songMessage));
 
 	// Channels' data
-	for(CHANNELINDEX chn = 0; chn < m_nChannels; chn++)
+	uint32 size = songHeader.channelNameLength;
+	for(auto &chn : ChnSettings)
 	{
-		ChnSettings[chn].nPan = std::min(static_cast<uint16>(file.ReadUint32LE()), uint16(256));
-		ChnSettings[chn].dwFlags.reset();
+		chn.nPan = std::min(static_cast<uint16>(file.ReadUint32LE()), uint16(256));
+		chn.dwFlags.reset();
 		uint32 flags = file.ReadUint32LE();
-		if(flags & 0x100) ChnSettings[chn].dwFlags.set(CHN_MUTE);
-		if(flags & 0x800) ChnSettings[chn].dwFlags.set(CHN_SURROUND);
-		ChnSettings[chn].nVolume = std::min(static_cast<uint16>(file.ReadUint32LE()), uint16(64));
-		file.ReadString<mpt::String::maybeNullTerminated>(ChnSettings[chn].szName, size);
+		if(flags & 0x100) chn.dwFlags.set(CHN_MUTE);
+		if(flags & 0x800) chn.dwFlags.set(CHN_SURROUND);
+		chn.nVolume = std::min(static_cast<uint8>(file.ReadUint32LE()), uint8(64));
+		file.ReadString<mpt::String::maybeNullTerminated>(chn.szName, size);
 	}
 
 	// Song mix plugins
@@ -225,12 +222,10 @@ bool CSoundFile::ReadITP(FileReader &file, ModLoadingFlags loadFlags)
 	m_MidiCfg.Sanitize();
 
 	// Song Instruments
-	m_nInstruments = static_cast<INSTRUMENTINDEX>(file.ReadUint32LE());
-	if(m_nInstruments >= MAX_INSTRUMENTS)
-	{
-		m_nInstruments = 0;
+	if(uint32 numIns = file.ReadUint32LE(); numIns < MAX_INSTRUMENTS)
+		m_nInstruments = static_cast<INSTRUMENTINDEX>(numIns);
+	else
 		return false;
-	}
 
 	// Instruments' paths
 	if(version <= 0x102)
@@ -250,7 +245,7 @@ bool CSoundFile::ReadITP(FileReader &file, ModLoadingFlags loadFlags)
 #ifdef MODPLUG_TRACKER
 		if(version <= 0x102)
 		{
-			instrPaths[ins] = mpt::PathString::FromLocaleSilent(path);
+			instrPaths[ins] = mpt::PathString::FromLocale(path);
 		} else
 #endif // MODPLUG_TRACKER
 		{
@@ -259,10 +254,10 @@ bool CSoundFile::ReadITP(FileReader &file, ModLoadingFlags loadFlags)
 #ifdef MODPLUG_TRACKER
 		if(const auto fileName = file.GetOptionalFileName(); fileName.has_value())
 		{
-			instrPaths[ins] = instrPaths[ins].RelativePathToAbsolute(fileName->GetPath());
+			instrPaths[ins] = mpt::RelativePathToAbsolute(instrPaths[ins], fileName->GetDirectoryWithDrive());
 		} else if(GetpModDoc() != nullptr)
 		{
-			instrPaths[ins] = instrPaths[ins].RelativePathToAbsolute(GetpModDoc()->GetPathNameMpt().GetPath());
+			instrPaths[ins] = mpt::RelativePathToAbsolute(instrPaths[ins], GetpModDoc()->GetPathNameMpt().GetDirectoryWithDrive());
 		}
 #endif // MODPLUG_TRACKER
 	}
@@ -307,15 +302,13 @@ bool CSoundFile::ReadITP(FileReader &file, ModLoadingFlags loadFlags)
 
 		// Pattern data
 		size_t numCommands = GetNumChannels() * numRows;
-
 		if(patternChunk.CanRead(sizeof(ITPModCommand) * numCommands))
 		{
-			ModCommand *target = Patterns[pat].GetpModCommand(0, 0);
-			while(numCommands-- != 0)
+			for(ModCommand &m : Patterns[pat])
 			{
 				ITPModCommand data;
 				patternChunk.ReadStruct(data);
-				*(target++) = data;
+				m = data;
 			}
 		}
 	}
@@ -328,7 +321,7 @@ bool CSoundFile::ReadITP(FileReader &file, ModLoadingFlags loadFlags)
 
 	// Read number of embedded samples - at most as many as there are real samples in a valid file
 	uint32 embeddedSamples = file.ReadUint32LE();
-	if(embeddedSamples > m_nSamples)
+	if(embeddedSamples > GetNumSamples())
 	{
 		return false;
 	}
@@ -361,7 +354,7 @@ bool CSoundFile::ReadITP(FileReader &file, ModLoadingFlags loadFlags)
 			continue;
 
 #ifdef MPT_EXTERNAL_SAMPLES
-		InputFile f(instrPaths[ins], SettingCacheCompleteFileBeforeLoading());
+		mpt::IO::InputFile f(instrPaths[ins], SettingCacheCompleteFileBeforeLoading());
 		FileReader instrFile = GetFileReader(f);
 		if(!ReadInstrumentFromFile(ins + 1, instrFile, true))
 		{
@@ -376,7 +369,7 @@ bool CSoundFile::ReadITP(FileReader &file, ModLoadingFlags loadFlags)
 	uint32 code = file.ReadUint32LE();
 
 	// Embed instruments' header [v1.01]
-	if(version >= 0x101 && (songFlags & ITP_ITPEMBEDIH) && code == MagicBE("EBIH"))
+	if(version >= 0x101 && (songHeader.flags & ITPSongHeader::ITP_ITPEMBEDIH) && code == MagicBE("EBIH"))
 	{
 		code = file.ReadUint32LE();
 
@@ -392,7 +385,7 @@ bool CSoundFile::ReadITP(FileReader &file, ModLoadingFlags loadFlags)
 				ins++;
 			} else
 			{
-				ReadExtendedInstrumentProperty(Instruments[ins], code, file);
+				ReadExtendedInstrumentProperty(mpt::as_span(&Instruments[ins], 1), code, file);
 			}
 
 			code = file.ReadUint32LE();
@@ -415,14 +408,14 @@ bool CSoundFile::ReadITP(FileReader &file, ModLoadingFlags loadFlags)
 	m_nMinPeriod = 8;
 
 	// Before OpenMPT 1.20.01.09, the MIDI macros were always read from the file, even if the "embed" flag was not set.
-	if(m_dwLastSavedWithVersion >= MPT_V("1.20.01.09") && !(songFlags & ITP_EMBEDMIDICFG))
+	if(m_dwLastSavedWithVersion >= MPT_V("1.20.01.09") && !(songHeader.flags & ITPSongHeader::ITP_EMBEDMIDICFG))
 	{
 		m_MidiCfg.Reset();
 	}
 
-	m_modFormat.formatName = U_("Impulse Tracker Project");
-	m_modFormat.type = U_("itp");
-	m_modFormat.madeWithTracker = U_("OpenMPT ") + mpt::ufmt::val(m_dwLastSavedWithVersion);
+	m_modFormat.formatName = UL_("Impulse Tracker Project");
+	m_modFormat.type = UL_("itp");
+	m_modFormat.madeWithTracker = UL_("OpenMPT ") + mpt::ufmt::val(m_dwLastSavedWithVersion);
 	m_modFormat.charset = mpt::Charset::Windows1252;
 
 	return true;
