@@ -126,7 +126,7 @@ bool CSoundFile::ReadMTM(FileReader &file, ModLoadingFlags loadFlags)
 	{
 		return false;
 	}
-	if(!file.CanRead(mpt::saturate_cast<FileReader::off_t>(GetHeaderMinimumAdditionalSize(fileHeader))))
+	if(!file.CanRead(mpt::saturate_cast<FileReader::pos_type>(GetHeaderMinimumAdditionalSize(fileHeader))))
 	{
 		return false;
 	}
@@ -135,13 +135,12 @@ bool CSoundFile::ReadMTM(FileReader &file, ModLoadingFlags loadFlags)
 		return true;
 	}
 
-	InitializeGlobals(MOD_TYPE_MTM);
+	InitializeGlobals(MOD_TYPE_MTM, fileHeader.numChannels);
 	m_songName = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, fileHeader.songName);
 	m_nSamples = fileHeader.numSamples;
-	m_nChannels = fileHeader.numChannels;
 	
-	m_modFormat.formatName = U_("MultiTracker");
-	m_modFormat.type = U_("mtm");
+	m_modFormat.formatName = UL_("MultiTracker");
+	m_modFormat.type = UL_("mtm");
 	m_modFormat.madeWithTracker = MPT_UFORMAT("MultiTracker {}.{}")(fileHeader.version >> 4, fileHeader.version & 0x0F);
 	m_modFormat.charset = mpt::Charset::CP437;
 
@@ -157,7 +156,6 @@ bool CSoundFile::ReadMTM(FileReader &file, ModLoadingFlags loadFlags)
 	// Setting Channel Pan Position
 	for(CHANNELINDEX chn = 0; chn < GetNumChannels(); chn++)
 	{
-		ChnSettings[chn].Reset();
 		ChnSettings[chn].nPan = ((fileHeader.panPos[chn] & 0x0F) << 4) + 8;
 	}
 
@@ -226,9 +224,7 @@ bool CSoundFile::ReadMTM(FileReader &file, ModLoadingFlags loadFlags)
 				}
 				if(cmd != 0 || param != 0)
 				{
-					m->command = cmd;
-					m->param = param;
-					ConvertModCommand(*m);
+					ConvertModCommand(*m, cmd, param);
 #ifdef MODPLUG_TRACKER
 					m->Convert(MOD_TYPE_MTM, MOD_TYPE_S3M, *this);
 #endif
@@ -254,13 +250,12 @@ bool CSoundFile::ReadMTM(FileReader &file, ModLoadingFlags loadFlags)
 		{
 			for(ROWINDEX row = 0; row < pattern.GetNumRows(); row++)
 			{
-				const auto rowBase = pattern.GetRow(row);
 				bool hasSpeedOnRow = false, hasTempoOnRow = false;
-				for(CHANNELINDEX chn = 0; chn < GetNumChannels(); chn++)
+				for(const ModCommand &m : pattern.GetRow(row))
 				{
-					if(rowBase[chn].command == CMD_SPEED)
+					if(m.command == CMD_SPEED)
 						hasSpeedOnRow = true;
-					else if(rowBase[chn].command == CMD_TEMPO)
+					else if(m.command == CMD_TEMPO)
 						hasTempoOnRow = true;
 				}
 				if(hasSpeedOnRow && hasTempoOnRow)
@@ -279,12 +274,11 @@ bool CSoundFile::ReadMTM(FileReader &file, ModLoadingFlags loadFlags)
 			{
 				for(ROWINDEX row = 0; row < pattern.GetNumRows(); row++)
 				{
-					const auto rowBase = pattern.GetRow(row);
-					for(CHANNELINDEX chn = 0; chn < GetNumChannels(); chn++)
+					for(const ModCommand &m : pattern.GetRow(row))
 					{
-						if(rowBase[chn].command == CMD_SPEED || rowBase[chn].command == CMD_TEMPO)
+						if(m.command == CMD_SPEED || m.command == CMD_TEMPO)
 						{
-							const bool writeTempo = rowBase[chn].command == CMD_SPEED;
+							const bool writeTempo = m.command == CMD_SPEED;
 							pattern.WriteEffect(EffectWriter(writeTempo ? CMD_TEMPO : CMD_SPEED, writeTempo ? 125 : 6).Row(row));
 							break;
 						}
