@@ -18,6 +18,7 @@
 #include "mpt/io/io_stdstream.hpp"
 #include "../common/mptFileIO.h"
 #ifdef MODPLUG_TRACKER
+#include "../mptrack/Moddoc.h"
 #include "../mptrack/TrackerSettings.h"
 #endif // MODPLUG_TRACKER
 #endif // MODPLUG_NO_FILESAVE
@@ -27,14 +28,15 @@
 OPENMPT_NAMESPACE_BEGIN
 
 
-void CSoundFile::S3MConvert(ModCommand &m, bool fromIT)
+void CSoundFile::S3MConvert(ModCommand &m, const uint8 command, const uint8 param, const bool fromIT)
 {
-	switch(m.command | 0x40)
+	m.param = param;
+	switch(command | 0x40)
 	{
 	case '@': m.command = (m.param ? CMD_DUMMY : CMD_NONE); break;
 	case 'A': m.command = CMD_SPEED; break;
 	case 'B': m.command = CMD_POSITIONJUMP; break;
-	case 'C': m.command = CMD_PATTERNBREAK; if (!fromIT) m.param = (m.param >> 4) * 10 + (m.param & 0x0F); break;
+	case 'C': m.command = CMD_PATTERNBREAK; if(!fromIT) m.param = static_cast<uint8>((m.param >> 4) * 10 + (m.param & 0x0F)); break;
 	case 'D': m.command = CMD_VOLUMESLIDE; break;
 	case 'E': m.command = CMD_PORTAMENTODOWN; break;
 	case 'F': m.command = CMD_PORTAMENTOUP; break;
@@ -73,18 +75,20 @@ void CSoundFile::S3MConvert(ModCommand &m, bool fromIT)
 
 #ifndef MODPLUG_NO_FILESAVE
 
-void CSoundFile::S3MSaveConvert(uint8 &command, uint8 &param, bool toIT, bool compatibilityExport) const
+void CSoundFile::S3MSaveConvert(const ModCommand &source, uint8 &command, uint8 &param, const bool toIT, const bool compatibilityExport) const
 {
+	command = 0;
+	param = source.param;
 	const bool extendedIT = !compatibilityExport && toIT;
-	switch(command)
+	switch(source.command)
 	{
 	case CMD_DUMMY:           command = (param ? '@' : 0); break;
 	case CMD_SPEED:           command = 'A'; break;
 	case CMD_POSITIONJUMP:    command = 'B'; break;
-	case CMD_PATTERNBREAK:    command = 'C'; if(!toIT) param = ((param / 10) << 4) + (param % 10); break;
+	case CMD_PATTERNBREAK:    command = 'C'; if(!toIT) param = static_cast<uint8>(((param / 10) << 4) + (param % 10)); break;
 	case CMD_VOLUMESLIDE:     command = 'D'; break;
-	case CMD_PORTAMENTODOWN:  command = 'E'; if (param >= 0xE0 && (GetType() & (MOD_TYPE_MOD | MOD_TYPE_XM))) param = 0xDF; break;
-	case CMD_PORTAMENTOUP:    command = 'F'; if (param >= 0xE0 && (GetType() & (MOD_TYPE_MOD | MOD_TYPE_XM))) param = 0xDF; break;
+	case CMD_PORTAMENTODOWN:  command = 'E'; if(param >= 0xE0 && (GetType() & (MOD_TYPE_MOD | MOD_TYPE_XM))) param = 0xDF; break;
+	case CMD_PORTAMENTOUP:    command = 'F'; if(param >= 0xE0 && (GetType() & (MOD_TYPE_MOD | MOD_TYPE_XM))) param = 0xDF; break;
 	case CMD_TONEPORTAMENTO:  command = 'G'; break;
 	case CMD_VIBRATO:         command = 'H'; break;
 	case CMD_TREMOR:          command = 'I'; break;
@@ -107,11 +111,11 @@ void CSoundFile::S3MSaveConvert(uint8 &command, uint8 &param, bool toIT, bool co
 		command = 'X';
 		if(toIT && !(GetType() & (MOD_TYPE_IT | MOD_TYPE_MPT | MOD_TYPE_XM | MOD_TYPE_MOD)))
 		{
-			if (param == 0xA4) { command = 'S'; param = 0x91; }
-			else if (param == 0x80) { param = 0xFF; }
-			else if (param < 0x80) { param <<= 1; }
+			if(param == 0xA4) { command = 'S'; param = 0x91; }
+			else if(param == 0x80) { param = 0xFF; }
+			else if(param < 0x80) { param <<= 1; }
 			else command = 0;
-		} else if (!toIT && (GetType() & (MOD_TYPE_IT | MOD_TYPE_MPT | MOD_TYPE_XM | MOD_TYPE_MOD)))
+		} else if(!toIT && (GetType() & (MOD_TYPE_IT | MOD_TYPE_MPT | MOD_TYPE_XM | MOD_TYPE_MOD)))
 		{
 			param >>= 1;
 		}
@@ -135,13 +139,11 @@ void CSoundFile::S3MSaveConvert(uint8 &command, uint8 &param, bool toIT, bool co
 		break;
 	case CMD_MODCMDEX:
 		{
-			ModCommand m;
-			m.command = CMD_MODCMDEX;
-			m.param = param;
-			m.ExtendedMODtoS3MEffect();
-			command = m.command;
-			param = m.param;
-			S3MSaveConvert(command, param, toIT, compatibilityExport);
+			ModCommand mConv;
+			mConv.command = CMD_MODCMDEX;
+			mConv.param = param;
+			mConv.ExtendedMODtoS3MEffect();
+			S3MSaveConvert(mConv, command, param, toIT, compatibilityExport);
 		}
 		return;
 	// Chars under 0x40 don't save properly, so map : to ] and # to [.
@@ -218,7 +220,7 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 	{
 		return false;
 	}
-	if(!file.CanRead(mpt::saturate_cast<FileReader::off_t>(GetHeaderMinimumAdditionalSize(fileHeader))))
+	if(!file.CanRead(mpt::saturate_cast<FileReader::pos_type>(GetHeaderMinimumAdditionalSize(fileHeader))))
 	{
 		return false;
 	}
@@ -227,10 +229,19 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		return true;
 	}
 
-	InitializeGlobals(MOD_TYPE_S3M);
+	InitializeGlobals(MOD_TYPE_S3M, fileHeader.GetNumChannels());
 	m_nMinPeriod = 64;
 	m_nMaxPeriod = 32767;
 
+	ReadOrderFromFile<uint8>(Order(), file, fileHeader.ordNum, 0xFF, 0xFE);
+
+	// Read sample header offsets
+	std::vector<uint16le> sampleOffsets;
+	file.ReadVector(sampleOffsets, fileHeader.smpNum);
+	// Read pattern offsets
+	std::vector<uint16le> patternOffsets;
+	file.ReadVector(patternOffsets, fileHeader.patNum);
+
 	// ST3 ignored Zxx commands, so if we find that a file was made with ST3, we should erase all MIDI macros.
 	bool keepMidiMacros = false;
 
@@ -239,78 +250,102 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 	bool nonCompatTracker = false;
 	bool isST3 = false;
 	bool isSchism = false;
+	const bool usePanningTable = fileHeader.usePanningTable == S3MFileHeader::idPanning;
+	const bool offsetsAreCanonical = !patternOffsets.empty() && !sampleOffsets.empty() && patternOffsets[0] > sampleOffsets[0];
 	const int32 schismDateVersion = SchismTrackerEpoch + ((fileHeader.cwtv == 0x4FFF) ? fileHeader.reserved2 : (fileHeader.cwtv - 0x4050));
+	uint32 editTimer = 0;
 	switch(fileHeader.cwtv & S3MFileHeader::trackerMask)
 	{
 	case S3MFileHeader::trkAkord & S3MFileHeader::trackerMask:
 		if(fileHeader.cwtv == S3MFileHeader::trkAkord)
-			madeWithTracker = U_("Akord");
+			madeWithTracker = UL_("Akord");
 		break;
 	case S3MFileHeader::trkScreamTracker:
-		if(fileHeader.cwtv == S3MFileHeader::trkST3_20 && fileHeader.special == 0 && (fileHeader.ordNum & 0x0F) == 0 && fileHeader.ultraClicks == 0 && (fileHeader.flags & ~0x50) == 0)
+		if(!memcmp(&fileHeader.reserved2, "SCLUB2.0", 8))
 		{
-			// MPT and OpenMPT before 1.17.03.02 - Simply keep default (filter) MIDI macros
-			if((fileHeader.masterVolume & 0x80) != 0)
+			madeWithTracker = UL_("Sound Club 2");
+		} else if(fileHeader.cwtv == S3MFileHeader::trkST3_20 && fileHeader.special == 0 && (fileHeader.ordNum & 0x01) == 0 && fileHeader.ultraClicks == 0 && (fileHeader.flags & ~0x50) == 0 && usePanningTable && offsetsAreCanonical)
+		{
+			// Canonical offset check avoids mis-detection of an automatic conversion of Vic's "Paper" demo track
+			if((fileHeader.ordNum & 0x0F) == 0)
 			{
-				m_dwLastSavedWithVersion = MPT_V("1.16.00.00");
-				madeWithTracker = U_("ModPlug Tracker / OpenMPT 1.17");
-			} else
+				// MPT and OpenMPT before 1.17.03.02 - Simply keep default (filter) MIDI macros
+				if((fileHeader.masterVolume & 0x80) != 0)
+				{
+					m_dwLastSavedWithVersion = MPT_V("1.16");
+					madeWithTracker = UL_("ModPlug Tracker / OpenMPT 1.17");
+				} else
+				{
+					// MPT 1.0 alpha5 doesn't set the stereo flag, but MPT 1.0 alpha6 does.
+					m_dwLastSavedWithVersion = MPT_V("1.00.00.A0");
+					madeWithTracker = UL_("ModPlug Tracker 1.0 alpha");
+				}
+			} else if((fileHeader.masterVolume & 0x80) != 0)
 			{
-				// MPT 1.0 alpha5 doesn't set the stereo flag, but MPT 1.0 beta1 does.
-				m_dwLastSavedWithVersion = MPT_V("1.00.00.00");
-				madeWithTracker = U_("ModPlug Tracker 1.0 alpha");
+				madeWithTracker = UL_("Schism Tracker");
 			}
 			keepMidiMacros = true;
 			nonCompatTracker = true;
 			m_playBehaviour.set(kST3LimitPeriod);
-		} else if(fileHeader.cwtv == S3MFileHeader::trkST3_20 && fileHeader.special == 0 && fileHeader.ultraClicks == 0 && fileHeader.flags == 0 && fileHeader.usePanningTable == 0)
+		} else if(fileHeader.cwtv == S3MFileHeader::trkST3_20 && fileHeader.special == 0 && fileHeader.ultraClicks == 0 && fileHeader.flags == 0 && !usePanningTable)
 		{
-			madeWithTracker = U_("Velvet Studio");
+			if(fileHeader.globalVol == 64 && fileHeader.masterVolume == 48)
+				madeWithTracker = UL_("PlayerPRO");
+			else  // Always stereo
+				madeWithTracker = UL_("Velvet Studio");
+		} else if(fileHeader.cwtv == S3MFileHeader::trkST3_20 && fileHeader.special == 0 && fileHeader.ultraClicks == 0 && fileHeader.flags == 8 && !usePanningTable)
+		{
+			madeWithTracker = UL_("Impulse Tracker < 1.03");  // Not sure if 1.02 saves like this as I don't have it
 		} else
 		{
 			// ST3.20 should only ever write ultra-click values 16, 24 and 32 (corresponding to 8, 12 and 16 in the GUI), ST3.01/3.03 should only write 0,
 			// though several ST3.01/3.03 files with ultra-click values of 16 have been found as well.
 			// However, we won't fingerprint these values here as it's unlikely that there is any other tracker out there disguising as ST3 and using a strange ultra-click value.
 			// Also, re-saving a file with a strange ultra-click value in ST3 doesn't fix this value unless the user manually changes it, or if it's below 16.
-			madeWithTracker = U_("Scream Tracker");
-			formatTrackerStr = true;
 			isST3 = true;
+			if(fileHeader.cwtv == S3MFileHeader::trkST3_20)
+			{
+				// 3.21 writes the version number as 3.20. There is no known way to differentiate between the two.
+				madeWithTracker = UL_("Scream Tracker 3.20 - 3.21");
+			} else
+			{
+				madeWithTracker = UL_("Scream Tracker");
+				formatTrackerStr = true;
+			}
 		}
 		break;
 	case S3MFileHeader::trkImagoOrpheus:
-		madeWithTracker = U_("Imago Orpheus");
-		formatTrackerStr = true;
+		formatTrackerStr = (fileHeader.cwtv != S3MFileHeader::trkPlayerPRO);
+		if(formatTrackerStr)
+			madeWithTracker = UL_("Imago Orpheus");
+		else
+			madeWithTracker = UL_("PlayerPRO");
 		nonCompatTracker = true;
 		break;
 	case S3MFileHeader::trkImpulseTracker:
-		if(fileHeader.cwtv <= S3MFileHeader::trkIT2_14)
-		{
-			madeWithTracker = U_("Impulse Tracker");
-			formatTrackerStr = true;
-		} else
-		{
-			madeWithTracker = MPT_UFORMAT("Impulse Tracker 2.14p{}")(fileHeader.cwtv - S3MFileHeader::trkIT2_14);
-		}
+		if(fileHeader.cwtv == S3MFileHeader::trkIT1_old)
+			madeWithTracker = UL_("Impulse Tracker 1.03");  // Could also be 1.02, maybe? I don't have that one
+		else
+			madeWithTracker = GetImpulseTrackerVersion(fileHeader.cwtv, 0);
+
 		if(fileHeader.cwtv >= S3MFileHeader::trkIT2_07 && fileHeader.reserved3 != 0)
 		{
 			// Starting from version 2.07, IT stores the total edit time of a module in the "reserved" field
-			uint32 editTime = DecodeITEditTimer(fileHeader.cwtv, fileHeader.reserved3);
-
-			FileHistory hist;
-			hist.openTime = static_cast<uint32>(editTime * (HISTORY_TIMER_PRECISION / 18.2));
-			m_FileHistory.push_back(hist);
+			editTimer = DecodeITEditTimer(fileHeader.cwtv, fileHeader.reserved3);
 		}
 		nonCompatTracker = true;
 		m_playBehaviour.set(kPeriodsAreHertz);
 		m_playBehaviour.set(kITRetrigger);
 		m_playBehaviour.set(kITShortSampleRetrig);
 		m_playBehaviour.set(kST3SampleSwap);  // Not exactly like ST3, but close enough
+		m_playBehaviour.set(kITPortaNoNote);
+		m_playBehaviour.set(kITPortamentoSwapResetsPos);
 		m_nMinPeriod = 1;
 		break;
 	case S3MFileHeader::trkSchismTracker:
 		if(fileHeader.cwtv == S3MFileHeader::trkBeRoTrackerOld)
 		{
-			madeWithTracker = U_("BeRoTracker");
+			madeWithTracker = UL_("BeRoTracker");
 			m_playBehaviour.set(kST3LimitPeriod);
 		} else
 		{
@@ -321,32 +356,56 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 				m_playBehaviour.set(kPeriodsAreHertz);
 			if(schismDateVersion >= SchismVersionFromDate<2016, 05, 13>::date)
 				m_playBehaviour.set(kITShortSampleRetrig);
+			m_playBehaviour.reset(kST3TonePortaWithAdlibNote);
+
+			if(fileHeader.cwtv == (S3MFileHeader::trkSchismTracker | 0xFFF) && fileHeader.reserved3 != 0)
+			{
+				// Added in commit 6c4b71f10d4e0bf202dddfa8bd781de510b8bc0b
+				editTimer = fileHeader.reserved3;
+			}
 		}
 		nonCompatTracker = true;
 		break;
 	case S3MFileHeader::trkOpenMPT:
-		if(fileHeader.cwtv != S3MFileHeader::trkGraoumfTracker)
+		if((fileHeader.cwtv & 0xFF00) == S3MFileHeader::trkNESMusa)
+		{
+			madeWithTracker = UL_("NESMusa");
+			formatTrackerStr = true;
+		} else if(fileHeader.reserved2 == 0 && fileHeader.ultraClicks == 16 && fileHeader.channels[1] != 1)
+		{
+			// Liquid Tracker's ID clashes with OpenMPT's.
+			// OpenMPT started writing full version information with OpenMPT 1.29 and later changed the ultraClicks value from 8 to 16.
+			// Liquid Tracker writes an ultraClicks value of 16.
+			// So we assume that a file was saved with Liquid Tracker if the reserved fields are 0 and ultraClicks is 16.
+			madeWithTracker = UL_("Liquid Tracker");
+			formatTrackerStr = true;
+		} else if(fileHeader.cwtv != S3MFileHeader::trkGraoumfTracker)
 		{
 			uint32 mptVersion = (fileHeader.cwtv & S3MFileHeader::versionMask) << 16;
 			if(mptVersion >= 0x01'29'00'00)
+			{
 				mptVersion |= fileHeader.reserved2;
+				// Added in OpenMPT 1.32.00.31
+				if(fileHeader.reserved3 != 0)
+					editTimer = fileHeader.reserved3;
+			}
 			m_dwLastSavedWithVersion = Version(mptVersion);
-			madeWithTracker = U_("OpenMPT ") + mpt::ufmt::val(m_dwLastSavedWithVersion);
+			madeWithTracker = UL_("OpenMPT ") + mpt::ufmt::val(m_dwLastSavedWithVersion);
 		} else
 		{
-			madeWithTracker = U_("Graoumf Tracker");
+			madeWithTracker = UL_("Graoumf Tracker");
 		}
 		break; 
 	case S3MFileHeader::trkBeRoTracker:
-		madeWithTracker = U_("BeRoTracker");
+		madeWithTracker = UL_("BeRoTracker");
 		m_playBehaviour.set(kST3LimitPeriod);
 		break;
 	case S3MFileHeader::trkCreamTracker:
-		madeWithTracker = U_("CreamTracker");
+		madeWithTracker = UL_("CreamTracker");
 		break;
 	default:
 		if(fileHeader.cwtv == S3MFileHeader::trkCamoto)
-			madeWithTracker = U_("Camoto");
+			madeWithTracker = UL_("Camoto");
 		break;
 	}
 	if(formatTrackerStr)
@@ -354,8 +413,16 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		madeWithTracker = MPT_UFORMAT("{} {}.{}")(madeWithTracker, (fileHeader.cwtv & 0xF00) >> 8, mpt::ufmt::hex0<2>(fileHeader.cwtv & 0xFF));
 	}
 
-	m_modFormat.formatName = U_("Scream Tracker 3");
-	m_modFormat.type = U_("s3m");
+	// IT edit timer
+	if(editTimer != 0)
+	{
+		FileHistory hist;
+		hist.openTime = static_cast<uint32>(editTimer * (HISTORY_TIMER_PRECISION / 18.2));
+		m_FileHistory.push_back(hist);
+	}
+
+	m_modFormat.formatName = UL_("Scream Tracker 3");
+	m_modFormat.type = UL_("s3m");
 	m_modFormat.madeWithTracker = std::move(madeWithTracker);
 	m_modFormat.charset = m_dwLastSavedWithVersion ? mpt::Charset::Windows1252 : mpt::Charset::CP437;
 
@@ -369,6 +436,11 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		m_playBehaviour.reset(kST3OffsetWithoutInstrument);
 		m_playBehaviour.reset(kApplyUpperPeriodLimit);
 	}
+	if(fileHeader.cwtv <= S3MFileHeader::trkST3_01)
+	{
+		// This broken behaviour is not present in ST3.01
+		m_playBehaviour.reset(kST3TonePortaWithAdlibNote);
+	}
 
 	if((fileHeader.cwtv & S3MFileHeader::trackerMask) > S3MFileHeader::trkScreamTracker)
 	{
@@ -396,26 +468,24 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		m_SongFlags.set(SONG_FASTVOLSLIDES);
 	}
 
-	// Speed
-	m_nDefaultSpeed = fileHeader.speed;
-	if(m_nDefaultSpeed == 0 || (m_nDefaultSpeed == 255 && isST3))
-	{
-		// Even though ST3 accepts the command AFF as expected, it mysteriously fails to load a default speed of 255...
-		m_nDefaultSpeed = 6;
-	}
+	// Even though ST3 accepts the command AFF as expected, it mysteriously fails to load a default speed of 255...
+	if(fileHeader.speed == 0 || (fileHeader.speed == 255 && isST3))
+		Order().SetDefaultSpeed(6);
+	else
+		Order().SetDefaultSpeed(fileHeader.speed);
 
-	// Tempo
-	m_nDefaultTempo.Set(fileHeader.tempo);
+	// ST3 also fails to load an otherwise valid default tempo of 32...
 	if(fileHeader.tempo < 33)
-	{
-		// ST3 also fails to load an otherwise valid default tempo of 32...
-		m_nDefaultTempo.Set(isST3 ? 125 : 32);
-	}
+		Order().SetDefaultTempoInt(isST3 ? 125 : 32);
+	else
+		Order().SetDefaultTempoInt(fileHeader.tempo);
 
 	// Global Volume
 	m_nDefaultGlobalVolume = std::min(fileHeader.globalVol.get(), uint8(64)) * 4u;
 	// The following check is probably not very reliable, but it fixes a few tunes, e.g.
-	// DARKNESS.S3M by Purple Motion (ST 3.00) and "Image of Variance" by C.C.Catch (ST 3.01):
+	// DARKNESS.S3M by Purple Motion (ST 3.00) and "Image of Variance" by C.C.Catch (ST 3.01).
+	// Note that even ST 3.01b imports these files with a global volume of 0,
+	// so it's not clear if these files ever played "as intended" in any ST3 versions (I don't have any older ST3 versions).
 	if(m_nDefaultGlobalVolume == 0 && fileHeader.cwtv < S3MFileHeader::trkST3_20)
 	{
 		m_nDefaultGlobalVolume = MAX_GLOBAL_VOLUME;
@@ -432,33 +502,27 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 	else
 		m_nSamplePreAmp = std::max(fileHeader.masterVolume & 0x7F, 0x10);  // Bit 7 = Stereo (we always use stereo)
 
-	const bool isStereo = (fileHeader.masterVolume & 0x80) != 0 || m_dwLastSavedWithVersion;
-	if(!isStereo)
-		m_nSamplePreAmp = Util::muldivr_unsigned(m_nSamplePreAmp, 8, 11);
-
 	// Approximately as loud as in DOSBox and a real SoundBlaster 16
 	m_nVSTiVolume = 36;
 	if(isSchism && schismDateVersion < SchismVersionFromDate<2018, 11, 12>::date)
 		m_nVSTiVolume = 64;
 
-	// Channel setup
-	m_nChannels = 4;
-	std::bitset<32> isAdlibChannel;
-	for(CHANNELINDEX i = 0; i < 32; i++)
+	const bool isStereo = (fileHeader.masterVolume & 0x80) != 0 || m_dwLastSavedWithVersion;
+	if(!isStereo)
 	{
-		ChnSettings[i].Reset();
+		m_nSamplePreAmp = Util::muldivr_unsigned(m_nSamplePreAmp, 8, 11);
+		m_nVSTiVolume = Util::muldivr_unsigned(m_nVSTiVolume, 8, 11);
+	}
 
+	// Channel setup
+	std::bitset<32> isAdlibChannel;
+	for(CHANNELINDEX i = 0; i < GetNumChannels(); i++)
+	{
 		uint8 ctype = fileHeader.channels[i] & ~0x80;
-		if(fileHeader.channels[i] != 0xFF)
-		{
-			m_nChannels = i + 1;
-			if(isStereo)
-				ChnSettings[i].nPan = (ctype & 8) ? 0xCC : 0x33;	// 200 : 56
-		}
+		if(fileHeader.channels[i] != 0xFF && isStereo)
+			ChnSettings[i].nPan = (ctype & 8) ? 0xCC : 0x33;  // 200 : 56
 		if(fileHeader.channels[i] & 0x80)
-		{
 			ChnSettings[i].dwFlags = CHN_MUTE;
-		}
 		if(ctype >= 16 && ctype <= 29)
 		{
 			// Adlib channel - except for OpenMPT 1.19 and older, which would write wrong channel types for PCM channels 16-32.
@@ -467,37 +531,33 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 			isAdlibChannel[i] = true;
 		}
 	}
-	if(m_nChannels < 1)
-	{
-		m_nChannels = 1;
-	}
-
-	ReadOrderFromFile<uint8>(Order(), file, fileHeader.ordNum, 0xFF, 0xFE);
-
-	// Read sample header offsets
-	std::vector<uint16le> sampleOffsets;
-	file.ReadVector(sampleOffsets, fileHeader.smpNum);
-	// Read pattern offsets
-	std::vector<uint16le> patternOffsets;
-	file.ReadVector(patternOffsets, fileHeader.patNum);
 
 	// Read extended channel panning
-	if(fileHeader.usePanningTable == S3MFileHeader::idPanning)
+	if(usePanningTable)
 	{
-		uint8 pan[32];
-		file.ReadArray(pan);
-		for(CHANNELINDEX i = 0; i < 32; i++)
+		bool hasChannelsWithoutPanning = false;
+		const auto pan = file.ReadArray<uint8, 32>();
+		for(CHANNELINDEX i = 0; i < GetNumChannels(); i++)
 		{
 			if((pan[i] & 0x20) != 0 && (!isST3 || !isAdlibChannel[i]))
-			{
-				ChnSettings[i].nPan = (static_cast<uint16>(pan[i] & 0x0F) * 256 + 8) / 15;
-			}
+				ChnSettings[i].nPan = static_cast<uint16>((static_cast<uint16>(pan[i] & 0x0F) * 256 + 8) / 15u);
+			else if(pan[i] < 0x10)
+				hasChannelsWithoutPanning = true;
+		}
+		if(GetNumChannels() < 32 && m_dwLastSavedWithVersion == MPT_V("1.16"))
+		{
+			// MPT 1.0 alpha 6 up to 1.16.203 set the panning bit for all channels, regardless of whether they are used or not.
+			// Note: Schism Tracker fixed the same bug in git commit f21fe8bcae8b6dde2df27ede4ac9fe563f91baff
+			if(hasChannelsWithoutPanning)
+				m_modFormat.madeWithTracker = UL_("ModPlug Tracker 1.16 / OpenMPT 1.17");
+			else
+				m_modFormat.madeWithTracker = UL_("ModPlug Tracker");
 		}
 	}
 
 	// Reading sample headers
 	m_nSamples = std::min(static_cast<SAMPLEINDEX>(fileHeader.smpNum), static_cast<SAMPLEINDEX>(MAX_SAMPLES - 1));
-	bool anySamples = false;
+	bool anySamples = false, anyADPCM = false;
 	uint16 gusAddresses = 0;
 	for(SAMPLEINDEX smp = 0; smp < m_nSamples; smp++)
 	{
@@ -509,34 +569,47 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		}
 
 		sampleHeader.ConvertToMPT(Samples[smp + 1], isST3);
-		m_szNames[smp + 1] = mpt::String::ReadBuf(mpt::String::nullTerminated, sampleHeader.name);
+		// Old ModPlug Tracker allowed to write into the last byte reserved for the null terminator
+		m_szNames[smp + 1] = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, sampleHeader.name);
 
 		if(sampleHeader.sampleType < S3MSampleHeader::typeAdMel)
 		{
-			const uint32 sampleOffset = sampleHeader.GetSampleOffset();
-			if((loadFlags & loadSampleData) && sampleHeader.length != 0 && file.Seek(sampleOffset))
+			if(sampleHeader.length != 0)
 			{
-				sampleHeader.GetSampleFormat((fileHeader.formatVersion == S3MFileHeader::oldVersion)).ReadSample(Samples[smp + 1], file);
+				SampleIO sampleIO = sampleHeader.GetSampleFormat((fileHeader.formatVersion == S3MFileHeader::oldVersion));
+				if((loadFlags & loadSampleData) && file.Seek(sampleHeader.GetSampleOffset()))
+					sampleIO.ReadSample(Samples[smp + 1], file);
 				anySamples = true;
+				if(sampleIO.GetEncoding() == SampleIO::ADPCM)
+					anyADPCM = true;
 			}
 			gusAddresses |= sampleHeader.gusAddress;
 		}
 	}
 
+	const bool useGUS = gusAddresses > 1;
 	if(isST3 && anySamples && !gusAddresses && fileHeader.cwtv != S3MFileHeader::trkST3_00)
 	{
 		// All Scream Tracker versions except for some probably early revisions of Scream Tracker 3.00 write GUS addresses. GUS support might not have existed at that point (1992).
 		// Hence if a file claims to be written with ST3 (but not ST3.00), but has no GUS addresses, we deduce that it must be written by some other software (e.g. some PSM -> S3M conversions)
 		isST3 = false;
-		MPT_UNUSED(isST3);		
-		m_modFormat.madeWithTracker = U_("Unknown");
+		m_modFormat.madeWithTracker = UL_("Unknown");
+		// Check these only after we are certain that it can't be ST3.01 because that version doesn't sanitize the ultraClicks value yet
+		if(fileHeader.cwtv == S3MFileHeader::trkST3_01 && fileHeader.ultraClicks == 0)
+		{
+			if(!(fileHeader.flags & ~(S3MFileHeader::fastVolumeSlides | S3MFileHeader::amigaLimits)) && (fileHeader.masterVolume & 0x80) && usePanningTable)
+				m_modFormat.madeWithTracker = UL_("UNMO3");
+			else if(!fileHeader.flags && fileHeader.globalVol == 48 && fileHeader.masterVolume == 176 && fileHeader.tempo == 150 && !usePanningTable)
+				m_modFormat.madeWithTracker = UL_("deMODifier");  // SoundSmith to S3M converter
+			else if(!fileHeader.flags && fileHeader.globalVol == 64 && (fileHeader.masterVolume & 0x7F) == 48 && fileHeader.speed == 6 && fileHeader.tempo == 125 && !usePanningTable)
+				m_modFormat.madeWithTracker = UL_("Kosmic To-S3M");  // MTM to S3M converter by Zab/Kosmic
+		}
 	} else if(isST3)
 	{
 		// Saving an S3M file in ST3 with the Gravis Ultrasound driver loaded will write a unique GUS memory address for each non-empty sample slot (and 0 for unused slots).
 		// Re-saving that file in ST3 with the SoundBlaster driver loaded will reset the GUS address for all samples to 0 (unused) or 1 (used).
 		// The first used sample will also have an address of 1 with the GUS driver.
 		// So this is a safe way of telling if the file was last saved with the GUS driver loaded or not if there's more than one sample.
-		const bool useGUS = gusAddresses > 1;
 		m_playBehaviour.set(kST3PortaSampleChange, useGUS);
 		m_playBehaviour.set(kST3SampleSwap, !useGUS);
 		m_playBehaviour.set(kITShortSampleRetrig, !useGUS);  // Only half the truth but close enough for now
@@ -545,6 +618,11 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		if(useGUS)
 			m_nSamplePreAmp = 48;
 	}
+	if(isST3)
+		m_playBehaviour.set(kS3MIgnoreCombinedFineSlides);
+
+	if(anyADPCM)
+		m_modFormat.madeWithTracker += UL_(" (ADPCM packed)");
 
 	// Try to find out if Zxx commands are supposed to be panning commands (PixPlay).
 	// Actually I am only aware of one module that uses this panning style, namely "Crawling Despair" by $volkraq
@@ -580,8 +658,9 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 
 		// Read pattern data
 		ROWINDEX row = 0;
-		PatternRow rowBase = Patterns[pat].GetRow(0);
+		auto rowBase = Patterns[pat].GetRow(0);
 
+		ModCommand dummy;
 		while(row < 64)
 		{
 			uint8 info = file.ReadUint8();
@@ -597,7 +676,6 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 			}
 
 			CHANNELINDEX channel = (info & s3mChannelMask);
-			ModCommand dummy;
 			ModCommand &m = (channel < GetNumChannels()) ? rowBase[channel] : dummy;
 
 			if(info & s3mNotePresent)
@@ -629,14 +707,22 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 			if(info & s3mEffectPresent)
 			{
 				const auto [command, param] = file.ReadArray<uint8, 2>();
-				m.command = command;
-				m.param = param;
-				S3MConvert(m, false);
+				S3MConvert(m, command, param, false);
 
 				if(m.command == CMD_S3MCMDEX && (m.param & 0xF0) == 0xA0 && fileHeader.cwtv < S3MFileHeader::trkST3_20)
 				{
-					// Convert old SAx panning to S8x (should only be found in PANIC.S3M by Purple Motion)
-					m.param = 0x80 | ((m.param & 0x0F) ^ 8);
+					// Convert the old messy SoundBlaster stereo control command (or an approximation of it, anyway)
+					const uint8 ctype = fileHeader.channels[channel] & 0x7F;
+					if(useGUS || ctype >= 0x10)
+						m.command = CMD_DUMMY;
+					else if(m.param == 0xA0 || m.param == 0xA2)  // Normal panning
+						m.param = (ctype & 8) ? 0x8C : 0x83;
+					else if(m.param == 0xA1 || m.param == 0xA3)  // Swap left / right channel
+						m.param = (ctype & 8) ? 0x83 : 0x8C;
+					else if(m.param <= 0xA7)  // Center
+						m.param = 0x88;
+					else
+						m.command = CMD_DUMMY;
 				} else if(m.command == CMD_MIDI)
 				{
 					// PixPlay panning test
@@ -651,7 +737,7 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 						else if(m.param > 0x08)
 							zxxCountRight++;
 					}
-				} else if(m.command == CMD_OFFSET && m.param == 0 && fileHeader.cwtv <= S3MFileHeader::trkST3_01)
+				} else if(m.command == CMD_OFFSET && m.param == 0 && isST3 && fileHeader.cwtv <= S3MFileHeader::trkST3_01)
 				{
 					// Offset command didn't have effect memory in ST3.01; fixed in ST3.03
 					m.command = CMD_DUMMY;
@@ -660,7 +746,7 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		}
 	}
 
-	if(pixPlayPanning && zxxCountLeft + zxxCountRight >= m_nChannels && (-zxxCountLeft + zxxCountRight) < static_cast<int>(m_nChannels))
+	if(pixPlayPanning && zxxCountLeft + zxxCountRight >= GetNumChannels() && (-zxxCountLeft + zxxCountRight) < static_cast<int>(GetNumChannels()))
 	{
 		// There are enough Zxx commands, so let's assume this was made to be played with PixPlay
 		Patterns.ForEachModCommand([](ModCommand &m)
@@ -687,7 +773,7 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 		0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,
 	};
 
-	if(m_nChannels == 0)
+	if(GetNumChannels() == 0)
 	{
 		return false;
 	}
@@ -757,12 +843,27 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 
 	// Song Variables
 	fileHeader.globalVol = static_cast<uint8>(std::min(m_nDefaultGlobalVolume / 4u, uint32(64)));
-	fileHeader.speed = static_cast<uint8>(Clamp(m_nDefaultSpeed, 1u, 254u));
-	fileHeader.tempo = static_cast<uint8>(Clamp(m_nDefaultTempo.GetInt(), 33u, 255u));
+	fileHeader.speed = static_cast<uint8>(Clamp(Order().GetDefaultSpeed(), 1u, 254u));
+	fileHeader.tempo = static_cast<uint8>(Clamp(Order().GetDefaultTempo().GetInt(), 33u, 255u));
 	fileHeader.masterVolume = static_cast<uint8>(Clamp(m_nSamplePreAmp, 16u, 127u) | 0x80);
 	fileHeader.ultraClicks = 16;
 	fileHeader.usePanningTable = S3MFileHeader::idPanning;
 
+	// IT edit timer
+	uint64 editTimer = 0;
+	for(const auto &mptHistory : GetFileHistory())
+	{
+		editTimer += static_cast<uint32>(mptHistory.openTime * HISTORY_TIMER_PRECISION / 18.2);
+	}
+#ifdef MODPLUG_TRACKER
+	if(const auto modDoc = GetpModDoc(); modDoc != nullptr)
+	{
+		auto creationTime = modDoc->GetCreationTime();
+		editTimer += mpt::saturate_round<uint64>((mpt::Date::UnixAsSeconds(mpt::Date::UnixNow()) - mpt::Date::UnixAsSeconds(creationTime)) * HISTORY_TIMER_PRECISION);
+	}
+#endif  // MODPLUG_TRACKER
+	fileHeader.reserved3 = mpt::saturate_cast<uint32>(editTimer);
+
 	mpt::IO::Write(f, fileHeader);
 	Order().WriteAsByte(f, writeOrders);
 
@@ -818,7 +919,8 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 
 	// Write patterns
 	enum class S3MChannelType : uint8 { kUnused = 0, kPCM = 1, kAdlib = 2 };
-	FlagSet<S3MChannelType> channelType[32] = { S3MChannelType::kUnused };
+	std::array<FlagSet<S3MChannelType>, 32> channelType;
+	channelType.fill(S3MChannelType::kUnused);
 	bool globalCmdOnMutedChn = false;
 	for(PATTERNINDEX pat = 0; pat < writePatterns; pat++)
 	{
@@ -853,39 +955,32 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 					continue;
 				}
 
-				const PatternRow rowBase = Patterns[pat].GetRow(row);
+				const auto rowBase = Patterns[pat].GetRow(row);
 
 				CHANNELINDEX writeChannels = std::min(CHANNELINDEX(32), GetNumChannels());
+				bool writePatternBreak = (Patterns[pat].GetNumRows() < 64 && row + 1 == Patterns[pat].GetNumRows() && !Patterns[pat].RowHasJump(row));
 				for(CHANNELINDEX chn = 0; chn < writeChannels; chn++)
 				{
 					const ModCommand &m = rowBase[chn];
 
 					uint8 info = static_cast<uint8>(chn);
 					uint8 note = m.note;
-					ModCommand::VOLCMD volcmd = m.volcmd;
-					uint8 vol = m.vol;
-					uint8 command = m.command;
-					uint8 param = m.param;
 
 					if(note != NOTE_NONE || m.instr != 0)
 					{
 						info |= s3mNotePresent;
 
-						if(note == NOTE_NONE)
-						{
-							note = s3mNoteNone;
-						} else if(ModCommand::IsSpecialNote(note))
+						if(ModCommand::IsSpecialNote(note))
 						{
 							// Note Cut
 							note = s3mNoteOff;
-						} else if(note < 12 + NOTE_MIN)
+						} else if(note >= NOTE_MIN + 12 && note <= NOTE_MIN + 107)
 						{
-							// Too low
-							note = 0;
-						} else if(note <= NOTE_MAX)
+							note -= NOTE_MIN + 12;
+							note = static_cast<uint8>((note % 12) + ((note / 12) << 4));
+						} else
 						{
-							note -= (12 + NOTE_MIN);
-							note = (note % 12) + ((note / 12) << 4);
+							note = s3mNoteNone;
 						}
 
 						if(m.instr > 0 && m.instr <= GetNumSamples())
@@ -898,25 +993,24 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 						}
 					}
 
-					if(command == CMD_VOLUME)
-					{
-						command = CMD_NONE;
-						volcmd = VOLCMD_VOLUME;
-						vol = std::min(param, uint8(64));
-					}
-
-					if(volcmd == VOLCMD_VOLUME)
+					uint8 vol = std::min(m.vol, ModCommand::VOL(64));
+					if(m.volcmd == VOLCMD_VOLUME)
 					{
 						info |= s3mVolumePresent;
-					} else if(volcmd == VOLCMD_PANNING)
+					} else if(m.volcmd == VOLCMD_PANNING)
 					{
 						info |= s3mVolumePresent;
 						vol |= 0x80;
+					} else if(m.command == CMD_VOLUME)
+					{
+						info |= s3mVolumePresent;
+						vol = std::min(m.param, ModCommand::PARAM(64));
 					}
 
-					if(command != CMD_NONE)
+					uint8 command = 0, param = 0;
+					if(m.command != CMD_NONE && m.command != CMD_VOLUME)
 					{
-						S3MSaveConvert(command, param, false, true);
+						S3MSaveConvert(m, command, param, false, true);
 						if(command || param)
 						{
 							info |= s3mEffectPresent;
@@ -926,6 +1020,12 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 							}
 						}
 					}
+					if(writePatternBreak && !(info & s3mEffectPresent))
+					{
+						info |= s3mEffectPresent;
+						command = 'C' ^ 0x40;
+						writePatternBreak = false;
+					}
 
 					if(info & s3mAnyPresent)
 					{
