@@ -49,7 +49,7 @@ struct STMSampleHeader
 			&& mptSmp.nLoopEnd != 0xFFFF)
 		{
 			mptSmp.uFlags = CHN_LOOP;
-			mptSmp.nLoopEnd = std::min(mptSmp.nLoopEnd, mptSmp.nLength);
+			mptSmp.nLength = std::max(mptSmp.nLoopEnd, mptSmp.nLength);  // ST2 does not sanitize loop end, allow it to overflow into the next sample's data
 		}
 	}
 };
@@ -113,7 +113,7 @@ static bool ValidateSTMOrderList(ModSequence &order)
 	for(auto &pat : order)
 	{
 		if(pat == 99 || pat == 255)  // 99 is regular, sometimes a single 255 entry can be found too
-			pat = order.GetInvalidPatIndex();
+			pat = PATTERNINDEX_INVALID;
 		else if(pat > 63)
 			return false;
 	}
@@ -121,7 +121,7 @@ static bool ValidateSTMOrderList(ModSequence &order)
 }
 
 
-static void ConvertSTMCommand(ModCommand &m, const ROWINDEX row, const uint8 fileVerMinor, uint8 &newTempo, ORDERINDEX &breakPos, ROWINDEX &breakRow)
+static void ConvertSTMCommand(ModCommand &m, const uint8 command, const ROWINDEX row, const uint8 fileVerMinor, uint8 &newTempo, ORDERINDEX &breakPos, ROWINDEX &breakRow)
 {
 	static constexpr EffectCommand stmEffects[] =
 	{
@@ -132,7 +132,7 @@ static void ConvertSTMCommand(ModCommand &m, const ROWINDEX row, const uint8 fil
 		// KLMNO can be entered in the editor but don't do anything
 	};
 
-	m.command = stmEffects[m.command & 0x0F];
+	m.command = stmEffects[command & 0x0F];
 
 	switch(m.command)
 	{
@@ -145,7 +145,7 @@ static void ConvertSTMCommand(ModCommand &m, const ROWINDEX row, const uint8 fil
 		break;
 
 	case CMD_PATTERNBREAK:
-		m.param = (m.param & 0xF0) * 10 + (m.param & 0x0F);
+		m.param = static_cast<ModCommand::PARAM>((m.param & 0xF0) * 10 + (m.param & 0x0F));
 		if(breakPos != ORDERINDEX_INVALID && m.param == 0)
 		{
 			// Merge Bxx + C00 into just Bxx
@@ -177,7 +177,7 @@ static void ConvertSTMCommand(ModCommand &m, const ROWINDEX row, const uint8 fil
 
 	case CMD_SPEED:
 		if(fileVerMinor < 21)
-			m.param = ((m.param / 10u) << 4u) + m.param % 10u;
+			m.param = static_cast<ModCommand::PARAM>(((m.param / 10u) << 4u) + m.param % 10u);
 
 		if(!m.param)
 		{
@@ -191,7 +191,7 @@ static void ConvertSTMCommand(ModCommand &m, const ROWINDEX row, const uint8 fil
 		newTempo = m.param;
 		m.param >>= 4;
 #else
-		MPT_UNUSED_VARIABLE(newTempo);
+		MPT_UNREFERENCED_PARAMETER(newTempo);
 #endif // MODPLUG_TRACKER
 		break;
 
@@ -224,24 +224,38 @@ bool CSoundFile::ReadSTM(FileReader &file, ModLoadingFlags loadFlags)
 		return false;
 	if(!fileHeader.Validate())
 		return false;
-	if(!file.CanRead(mpt::saturate_cast<FileReader::off_t>(fileHeader.GetHeaderMinimumAdditionalSize())))
+	if(!file.CanRead(mpt::saturate_cast<FileReader::pos_type>(fileHeader.GetHeaderMinimumAdditionalSize())))
 		return false;
 	if(loadFlags == onlyVerifyHeader)
 		return true;
 
-	InitializeGlobals(MOD_TYPE_STM);
+	InitializeGlobals(MOD_TYPE_STM, 4);
 
 	m_songName = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, fileHeader.songname);
 
-	m_modFormat.formatName = U_("Scream Tracker 2");
-	m_modFormat.type = U_("stm");
-	m_modFormat.madeWithTracker = MPT_UFORMAT("Scream Tracker {}.{}")(fileHeader.verMajor, mpt::ufmt::dec0<2>(fileHeader.verMinor));
+	m_modFormat.formatName = UL_("Scream Tracker 2");
+	m_modFormat.type = UL_("stm");
 	m_modFormat.charset = mpt::Charset::CP437;
 
+	if(!std::memcmp(fileHeader.trackerName, "!Scream!", 8))
+	{
+		if(fileHeader.verMinor >= 21)
+			m_modFormat.madeWithTracker = UL_("Scream Tracker 2.2 - 2.3 or compatible");
+		else
+			m_modFormat.madeWithTracker = MPT_UFORMAT("Scream Tracker {}.{} or compatible")(fileHeader.verMajor, mpt::ufmt::dec0<2>(fileHeader.verMinor));
+	}
+	else if(!std::memcmp(fileHeader.trackerName, "BMOD2STM", 8))
+		m_modFormat.madeWithTracker = UL_("BMOD2STM");
+	else if(!std::memcmp(fileHeader.trackerName, "WUZAMOD!", 8))
+		m_modFormat.madeWithTracker = UL_("Wuzamod");
+	else if(!std::memcmp(fileHeader.trackerName, "SWavePro", 8))
+		m_modFormat.madeWithTracker = UL_("SoundWave Pro");
+	else
+		m_modFormat.madeWithTracker = UL_("Unknown");
+
 	m_playBehaviour.set(kST3SampleSwap);
 
 	m_nSamples = 31;
-	m_nChannels = 4;
 	m_nMinPeriod = 64;
 	m_nMaxPeriod = 0x7FFF;
 	
@@ -249,22 +263,15 @@ bool CSoundFile::ReadSTM(FileReader &file, ModLoadingFlags loadFlags)
 	
 	uint8 initTempo = fileHeader.initTempo;
 	if(fileHeader.verMinor < 21)
-		initTempo = ((initTempo / 10u) << 4u) + initTempo % 10u;
+		initTempo = static_cast<uint8>(((initTempo / 10u) << 4u) + initTempo % 10u);
 	if(initTempo == 0)
 		initTempo = 0x60;
 
-	m_nDefaultTempo = ConvertST2Tempo(initTempo);
-	m_nDefaultSpeed = initTempo >> 4;
+	Order().SetDefaultTempo(ConvertST2Tempo(initTempo));
+	Order().SetDefaultSpeed(initTempo >> 4);
 	if(fileHeader.verMinor > 10)
 		m_nDefaultGlobalVolume = std::min(fileHeader.globalVolume, uint8(64)) * 4u;
 
-	// Setting up channels
-	for(CHANNELINDEX chn = 0; chn < 4; chn++)
-	{
-		ChnSettings[chn].Reset();
-		ChnSettings[chn].nPan = (chn & 1) ? 0x40 : 0xC0;
-	}
-
 	// Read samples
 	uint16 sampleOffsets[31];
 	for(SAMPLEINDEX smp = 1; smp <= 31; smp++)
@@ -331,7 +338,7 @@ bool CSoundFile::ReadSTM(FileReader &file, ModLoadingFlags loadFlags)
 				if(note == 0xFE)
 					m->note = NOTE_NOTECUT;
 				else if(note < 0x60)
-					m->note = (note >> 4) * 12 + (note & 0x0F) + 36 + NOTE_MIN;
+					m->note = static_cast<ModCommand::NOTE>((note >> 4) * 12 + (note & 0x0F) + 36 + NOTE_MIN);
 
 				m->instr = insVol >> 3;
 				if(m->instr > 31)
@@ -346,9 +353,8 @@ bool CSoundFile::ReadSTM(FileReader &file, ModLoadingFlags loadFlags)
 					m->vol = vol;
 				}
 
-				m->command = volCmd & 0x0F;
 				m->param = cmdInf;
-				ConvertSTMCommand(*m, row, fileHeader.verMinor, newTempo, breakPos, breakRow);
+				ConvertSTMCommand(*m, volCmd & 0x0F, row, fileHeader.verMinor, newTempo, breakPos, breakRow);
 			}
 			if(newTempo != 0)
 			{
@@ -377,7 +383,7 @@ bool CSoundFile::ReadSTM(FileReader &file, ModLoadingFlags loadFlags)
 			// ST2 just plays random noise for samples with a default volume of 0
 			if(sample.nLength && sample.nVolume > 0)
 			{
-				FileReader::off_t sampleOffset = sampleOffsets[smp - 1] << 4;
+				FileReader::pos_type sampleOffset = sampleOffsets[smp - 1] << 4;
 				// acidlamb.stm has some bogus samples with sample offsets past EOF
 				if(sampleOffset > sizeof(STMFileHeader) && file.Seek(sampleOffset))
 				{
@@ -456,17 +462,16 @@ bool CSoundFile::ReadSTX(FileReader &file, ModLoadingFlags loadFlags)
 		return false;
 	if(!fileHeader.Validate())
 		return false;
-	if (!file.CanRead(mpt::saturate_cast<FileReader::off_t>(fileHeader.GetHeaderMinimumAdditionalSize())))
+	if (!file.CanRead(mpt::saturate_cast<FileReader::pos_type>(fileHeader.GetHeaderMinimumAdditionalSize())))
 		return false;
 	if(loadFlags == onlyVerifyHeader)
 		return true;
 
-	InitializeGlobals(MOD_TYPE_STM);
+	InitializeGlobals(MOD_TYPE_STM, 4);
 
 	m_songName = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, fileHeader.songName);
 
 	m_nSamples = fileHeader.numSamples;
-	m_nChannels = 4;
 	m_nMinPeriod = 64;
 	m_nMaxPeriod = 0x7FFF;
 
@@ -476,17 +481,10 @@ bool CSoundFile::ReadSTX(FileReader &file, ModLoadingFlags loadFlags)
 	if(initTempo == 0)
 		initTempo = 0x60;
 
-	m_nDefaultTempo = ConvertST2Tempo(initTempo);
-	m_nDefaultSpeed = initTempo >> 4;
+	Order().SetDefaultTempo(ConvertST2Tempo(initTempo));
+	Order().SetDefaultSpeed(initTempo >> 4);
 	m_nDefaultGlobalVolume = std::min(fileHeader.globalVolume, uint8(64)) * 4u;
 
-	// Setting up channels
-	for(CHANNELINDEX chn = 0; chn < 4; chn++)
-	{
-		ChnSettings[chn].Reset();
-		ChnSettings[chn].nPan = (chn & 1) ? 0x40 : 0xC0;
-	}
-
 	std::vector<uint16le> patternOffsets, sampleOffsets;
 	file.Seek(fileHeader.patTableOffset << 4);
 	file.ReadVector(patternOffsets, fileHeader.numPatterns);
@@ -594,9 +592,8 @@ bool CSoundFile::ReadSTX(FileReader &file, ModLoadingFlags loadFlags)
 			if(info & s3mEffectPresent)
 			{
 				const auto [command, param] = file.ReadArray<uint8, 2>();
-				m.command = command;
 				m.param = param;
-				ConvertSTMCommand(m, row, 0xFF, newTempo, breakPos, breakRow);
+				ConvertSTMCommand(m, command, row, 0xFF, newTempo, breakPos, breakRow);
 			}
 		}
 
@@ -606,8 +603,8 @@ bool CSoundFile::ReadSTX(FileReader &file, ModLoadingFlags loadFlags)
 		}
 	}
 
-	m_modFormat.formatName = U_("Scream Tracker Music Interface Kit");
-	m_modFormat.type = U_("stx");
+	m_modFormat.formatName = UL_("Scream Tracker Music Interface Kit");
+	m_modFormat.type = UL_("stx");
 	m_modFormat.charset = mpt::Charset::CP437;
 	m_modFormat.madeWithTracker = MPT_UFORMAT("STM2STX 1.{}")(formatVersion);
 
