@@ -10,7 +10,7 @@
 
 #pragma once
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
 
 
 OPENMPT_NAMESPACE_BEGIN
@@ -135,7 +135,7 @@ namespace MIDIEvents
 	};
 
 	// MIDI CC Names
-	extern const char* const MidiCCNames[MIDICC_end + 1];
+	extern const char* const MidiCCNames[MIDICC_end + 1];  // Charset::UTF8
 
 	// Build a generic MIDI event
 	uint32 Event(EventType eventType, uint8 midiChannel, uint8 dataByte1, uint8 dataByte2);
@@ -151,6 +151,8 @@ namespace MIDIEvents
 	uint32 NoteOn(uint8 midiChannel, uint8 note, uint8 velocity);
 	// Build a MIDI System Event
 	uint8 System(SystemEvent eventType);
+	// Build a MIDI Song Position Event
+	uint32 SongPosition(uint16 quarterNotes);
 
 	// Get MIDI channel from a MIDI event
 	uint8 GetChannelFromEvent(uint32 midiMsg);
