@@ -10,7 +10,7 @@
 
 #pragma once
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
 
 #include "MixerInterface.h"
 
@@ -18,30 +18,30 @@ OPENMPT_NAMESPACE_BEGIN
 
 namespace MixFuncTable
 {
-	// Table index:
-	//	[b1-b0]	format (8-bit-mono, 16-bit-mono, 8-bit-stereo, 16-bit-stereo)
-	//	[b2]	ramp
-	//	[b3]	filter
-	//	[b6-b4]	src type
+	// Table index bits:
+	// [b1-b0] format (8-bit-mono, 16-bit-mono, 8-bit-stereo, 16-bit-stereo)
+	// [b2]    ramp
+	// [b3]    filter
+	// [b6-b4] src type
 
 	// Sample type / processing type index
 	enum FunctionIndex
 	{
-		ndx16Bit		= 0x01,
-		ndxStereo		= 0x02,
-		ndxRamp			= 0x04,
-		ndxFilter		= 0x08,
+		ndx16Bit  = 0x01,
+		ndxStereo = 0x02,
+		ndxRamp   = 0x04,
+		ndxFilter = 0x08,
 	};
 
 	// SRC index
 	enum ResamplingIndex
 	{
-		ndxNoInterpolation	= 0x00,
-		ndxLinear			= 0x10,
-		ndxFastSinc			= 0x20,
-		ndxKaiser			= 0x30,
-		ndxFIRFilter		= 0x40,
-		ndxAmigaBlep		= 0x50,
+		ndxNoInterpolation = 0x00,
+		ndxLinear          = 0x10,
+		ndxFastSinc        = 0x20,
+		ndxKaiser          = 0x30,
+		ndxFIRFilter       = 0x40,
+		ndxAmigaBlep       = 0x50,
 	};
 
 	extern const MixFuncInterface Functions[6 * 16];
