@@ -23,8 +23,8 @@ void FloatToStereoMix(const float *pIn1, const float *pIn2, int32 *pOut, uint32
 {
 	for(uint32 i=0; i<nCount; ++i)
 	{
-		*pOut++ = (int)(*pIn1++ * _f2ic);
-		*pOut++ = (int)(*pIn2++ * _f2ic);
+		*pOut++ = static_cast<int>(*pIn1++ * _f2ic);
+		*pOut++ = static_cast<int>(*pIn2++ * _f2ic);
 	}
 }
 
@@ -33,26 +33,8 @@ void StereoMixToFloat(const int32 *pSrc, float *pOut1, float *pOut2, uint32 nCou
 {
 	for(uint32 i=0; i<nCount; ++i)
 	{
-		*pOut1++ = *pSrc++ * _i2fc;
-		*pOut2++ = *pSrc++ * _i2fc;
-	}
-}
-
-
-void FloatToMonoMix(const float *pIn, int32 *pOut, uint32 nCount, const float _f2ic)
-{
-	for(uint32 i=0; i<nCount; ++i)
-	{
-		*pOut++ = (int)(*pIn++ * _f2ic);
-	}
-}
-
-
-void MonoMixToFloat(const int32 *pSrc, float *pOut, uint32 nCount, const float _i2fc)
-{
-	for(uint32 i=0; i<nCount; ++i)
-	{
-		*pOut++ = *pSrc++ * _i2fc;
+		*pOut1++ = static_cast<float>(*pSrc++) * _i2fc;
+		*pOut2++ = static_cast<float>(*pSrc++) * _i2fc;
 	}
 }
 
@@ -159,6 +141,8 @@ void EndChannelOfs(ModChannel &chn, mixsample_t *pBuffer, uint32 nSamples)
 
 
 
+#ifndef MPT_INTMIXER
+
 void InterleaveStereo(const mixsample_t * MPT_RESTRICT inputL, const mixsample_t * MPT_RESTRICT inputR, mixsample_t * MPT_RESTRICT output, size_t numSamples)
 {
 	while(numSamples--)
@@ -178,6 +162,8 @@ void DeinterleaveStereo(const mixsample_t * MPT_RESTRICT input, mixsample_t * MP
 	}
 }
 
+#endif
+
 
 
 OPENMPT_NAMESPACE_END
