@@ -11,12 +11,15 @@
 
 #pragma once
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
 
 OPENMPT_NAMESPACE_BEGIN
 
-extern const MPT_UCHAR_TYPE NoteNamesSharp[12][4];
-extern const MPT_UCHAR_TYPE NoteNamesFlat[12][4];
+// Compute Bessel function Izero(y) using a series approximation
+double Izero(double y);
+
+extern const mpt::uchar NoteNamesSharp[12][4];
+extern const mpt::uchar NoteNamesFlat[12][4];
 
 extern const uint8 ImpulseTrackerPortaVolCmd[16];
 extern const uint16 ProTrackerPeriodTable[7*12];
@@ -36,6 +39,7 @@ extern const uint32 FineLinearSlideDownTable[16];
 extern const uint32 LinearSlideUpTable[256];
 extern const uint32 LinearSlideDownTable[256];
 extern const uint16 XMPanningTable[256];
+extern const std::array<uint8, 16> HisMastersNoiseMegaArp[16];
 
 extern const uint8 AutoVibratoIT2XM[8];
 extern const uint8 AutoVibratoXM2IT[8];
