@@ -12,11 +12,13 @@
 
 #ifndef NO_PLUGINS
 
-#include "../../common/version.h"
 #include "PluginManager.h"
 #include "PlugInterface.h"
+#include "../../common/version.h"
 
+#if defined(MPT_WITH_DMO)
 #include "mpt/uuid/guid.hpp"
+#endif // MPT_WITH_DMO
 #include "mpt/uuid/uuid.hpp"
 
 // Built-in plugins
@@ -44,6 +46,9 @@
 #ifdef MPT_WITH_VST
 #include "../../mptrack/Vstplug.h"
 #include "../../pluginBridge/BridgeWrapper.h"
+#ifdef MODPLUG_TRACKER
+#include "mpt/fs/fs.hpp"
+#endif // MODPLUG_TRACKER
 #endif // MPT_WITH_VST
 
 #if defined(MPT_WITH_DMO)
@@ -54,6 +59,7 @@
 
 #ifdef MODPLUG_TRACKER
 #include "../../mptrack/Mptrack.h"
+#include "../../mptrack/Reporting.h"
 #include "../../mptrack/TrackerSettings.h"
 #include "../../mptrack/AbstractVstEditor.h"
 #include "../../soundlib/AudioCriticalSection.h"
@@ -131,52 +137,6 @@ mpt::ustring VSTPluginLib::GetPluginArchName(uint8 arch)
 }
 
 
-mpt::ustring VSTPluginLib::GetPluginArchNameUser(uint8 arch)
-{
-	mpt::ustring result;
-	#if defined(MPT_WITH_WINDOWS10)
-		switch(arch)
-		{
-		case PluginArch_x86:
-			result = U_("x86 (32bit)");
-			break;
-		case PluginArch_amd64:
-			result = U_("amd64 (64bit)");
-			break;
-		case PluginArch_arm:
-			result = U_("arm (32bit)");
-			break;
-		case PluginArch_arm64:
-			result = U_("arm64 (64bit)");
-			break;
-		default:
-			result = U_("");
-			break;
-		}
-	#else // !MPT_WITH_WINDOWS10
-		switch(arch)
-		{
-		case PluginArch_x86:
-			result = U_("32-Bit");
-			break;
-		case PluginArch_amd64:
-			result = U_("64-Bit");
-			break;
-		case PluginArch_arm:
-			result = U_("32-Bit");
-			break;
-		case PluginArch_arm64:
-			result = U_("64-Bit");
-			break;
-		default:
-			result = U_("");
-			break;
-		}
-	#endif // MPT_WITH_WINDOWS10
-	return result;
-}
-
-
 uint8 VSTPluginLib::GetDllArch(bool fromCache) const
 {
 	// Built-in plugins are always native.
@@ -202,7 +162,7 @@ mpt::ustring VSTPluginLib::GetDllArchName(bool fromCache) const
 
 mpt::ustring VSTPluginLib::GetDllArchNameUser(bool fromCache) const
 {
-	return GetPluginArchNameUser(GetDllArch(fromCache));
+	return GetPluginArchName(GetDllArch(fromCache));
 }
 
 
@@ -231,23 +191,57 @@ void VSTPluginLib::WriteToCache() const
 {
 	SettingsContainer &cacheFile = theApp.GetPluginCache();
 
-	const std::string crcName = dllPath.ToUTF8();
-	const mpt::crc32 crc(crcName);
-	const mpt::ustring IDs = mpt::ufmt::HEX0<8>(pluginId1) + mpt::ufmt::HEX0<8>(pluginId2) + mpt::ufmt::HEX0<8>(crc.result());
-
-	mpt::PathString writePath = dllPath;
+	mpt::ustring pathStr;
 	if(theApp.IsPortableMode())
-	{
-		writePath = theApp.PathAbsoluteToInstallRelative(writePath);
-	}
+		pathStr = theApp.PathAbsoluteToInstallRelative(dllPath).ToUnicode();
+	else
+		pathStr = dllPath.ToUnicode();
 
-	cacheFile.Write<mpt::ustring>(cacheSection, writePath.ToUnicode(), IDs);
+	if(shellPluginID)
+		pathStr += U_("|") + mpt::ufmt::HEX0<8>(shellPluginID);
+
+	// CRC is used to distinguish plugins sharing the same IDs
+	const std::string crcName = mpt::ToCharset(mpt::Charset::UTF8, pathStr);
+	const mpt::crc32 crc(crcName);
+	const mpt::ustring IDs = mpt::ufmt::HEX0<8>(static_cast<uint32>(pluginId1)) + mpt::ufmt::HEX0<8>(static_cast<uint32>(pluginId2)) + mpt::ufmt::HEX0<8>(crc.result());
+
+	cacheFile.Write<mpt::ustring>(cacheSection, pathStr, IDs);
 	cacheFile.Write<CString>(cacheSection, IDs + U_(".Vendor"), vendor);
 	cacheFile.Write<int32>(cacheSection, IDs + U_(".Flags"), EncodeCacheFlags());
 }
 #endif // MODPLUG_TRACKER
 
 
+void VSTPluginLib::InsertPluginInstanceIntoList(IMixPlugin &pluginInstance)
+{
+	pluginInstance.m_pNext = pPluginsList;
+	if(pPluginsList)
+	{
+		pPluginsList->m_pPrev = &pluginInstance;
+	}
+	pPluginsList = &pluginInstance;
+}
+
+
+void VSTPluginLib::RemovePluginInstanceFromList(IMixPlugin &pluginInstance)
+{
+	if(pPluginsList == &pluginInstance)
+	{
+		pPluginsList = pluginInstance.m_pNext;
+	}
+	if(pluginInstance.m_pNext)
+	{
+		pluginInstance.m_pNext->m_pPrev = pluginInstance.m_pPrev;
+	}
+	if(pluginInstance.m_pPrev)
+	{
+		pluginInstance.m_pPrev->m_pNext = pluginInstance.m_pNext;
+	}
+	pluginInstance.m_pPrev = nullptr;
+	pluginInstance.m_pNext = nullptr;
+}
+
+
 bool CreateMixPluginProc(SNDMIXPLUGIN &mixPlugin, CSoundFile &sndFile)
 {
 #ifdef MODPLUG_TRACKER
@@ -283,49 +277,45 @@ CVstPluginManager::CVstPluginManager()
 		VSTPluginLib::CreateProc createProc;
 		const char *filename, *name;
 		uint32 pluginId1, pluginId2;
-		VSTPluginLib::PluginCategory category;
+		PluginCategory category;
 		bool isInstrument, isOurs;
 	} BuiltInPlugins[] =
 	{
 		// DirectX Media Objects Emulation
-		{ DMO::Chorus::Create,      "{EFE6629C-81F7-4281-BD91-C9D604A95AF6}", "Chorus",      kDmoMagic, 0xEFE6629C, VSTPluginLib::catDMO, false, false },
-		{ DMO::Compressor::Create,  "{EF011F79-4000-406D-87AF-BFFB3FC39D57}", "Compressor",  kDmoMagic, 0xEF011F79, VSTPluginLib::catDMO, false, false },
-		{ DMO::Distortion::Create,  "{EF114C90-CD1D-484E-96E5-09CFAF912A21}", "Distortion",  kDmoMagic, 0xEF114C90, VSTPluginLib::catDMO, false, false },
-		{ DMO::Echo::Create,        "{EF3E932C-D40B-4F51-8CCF-3F98F1B29D5D}", "Echo",        kDmoMagic, 0xEF3E932C, VSTPluginLib::catDMO, false, false },
-		{ DMO::Flanger::Create,     "{EFCA3D92-DFD8-4672-A603-7420894BAD98}", "Flanger",     kDmoMagic, 0xEFCA3D92, VSTPluginLib::catDMO, false, false },
-		{ DMO::Gargle::Create,      "{DAFD8210-5711-4B91-9FE3-F75B7AE279BF}", "Gargle",      kDmoMagic, 0xDAFD8210, VSTPluginLib::catDMO, false, false },
-		{ DMO::I3DL2Reverb::Create, "{EF985E71-D5C7-42D4-BA4D-2D073E2E96F4}", "I3DL2Reverb", kDmoMagic, 0xEF985E71, VSTPluginLib::catDMO, false, false },
-		{ DMO::ParamEq::Create,     "{120CED89-3BF4-4173-A132-3CB406CF3231}", "ParamEq",     kDmoMagic, 0x120CED89, VSTPluginLib::catDMO, false, false },
-		{ DMO::WavesReverb::Create, "{87FC0268-9A55-4360-95AA-004A1D9DE26C}", "WavesReverb", kDmoMagic, 0x87FC0268, VSTPluginLib::catDMO, false, false },
+		{ DMO::Chorus::Create,      "{EFE6629C-81F7-4281-BD91-C9D604A95AF6}", "Chorus",      kDmoMagic, 0xEFE6629C, PluginCategory::DMO, false, false },
+		{ DMO::Compressor::Create,  "{EF011F79-4000-406D-87AF-BFFB3FC39D57}", "Compressor",  kDmoMagic, 0xEF011F79, PluginCategory::DMO, false, false },
+		{ DMO::Distortion::Create,  "{EF114C90-CD1D-484E-96E5-09CFAF912A21}", "Distortion",  kDmoMagic, 0xEF114C90, PluginCategory::DMO, false, false },
+		{ DMO::Echo::Create,        "{EF3E932C-D40B-4F51-8CCF-3F98F1B29D5D}", "Echo",        kDmoMagic, 0xEF3E932C, PluginCategory::DMO, false, false },
+		{ DMO::Flanger::Create,     "{EFCA3D92-DFD8-4672-A603-7420894BAD98}", "Flanger",     kDmoMagic, 0xEFCA3D92, PluginCategory::DMO, false, false },
+		{ DMO::Gargle::Create,      "{DAFD8210-5711-4B91-9FE3-F75B7AE279BF}", "Gargle",      kDmoMagic, 0xDAFD8210, PluginCategory::DMO, false, false },
+		{ DMO::I3DL2Reverb::Create, "{EF985E71-D5C7-42D4-BA4D-2D073E2E96F4}", "I3DL2Reverb", kDmoMagic, 0xEF985E71, PluginCategory::DMO, false, false },
+		{ DMO::ParamEq::Create,     "{120CED89-3BF4-4173-A132-3CB406CF3231}", "ParamEq",     kDmoMagic, 0x120CED89, PluginCategory::DMO, false, false },
+		{ DMO::WavesReverb::Create, "{87FC0268-9A55-4360-95AA-004A1D9DE26C}", "WavesReverb", kDmoMagic, 0x87FC0268, PluginCategory::DMO, false, false },
 		// First (inaccurate) Flanger implementation (will be chosen based on library name, shares ID1 and ID2 with regular Flanger)
-		{ DMO::Flanger::CreateLegacy, "{EFCA3D92-DFD8-4672-A603-7420894BAD98}", "Flanger (Legacy)", kDmoMagic, 0xEFCA3D92, VSTPluginLib::catHidden, false, false },
+		{ DMO::Flanger::CreateLegacy, "{EFCA3D92-DFD8-4672-A603-7420894BAD98}", "Flanger (Legacy)", kDmoMagic, 0xEFCA3D92, PluginCategory::Hidden, false, false },
 		// DigiBooster Pro Echo DSP
-		{ DigiBoosterEcho::Create, "", "DigiBooster Pro Echo", MagicLE("DBM0"), MagicLE("Echo"), VSTPluginLib::catRoomFx, false, true },
+		{ DigiBoosterEcho::Create, "", "DigiBooster Pro Echo", MagicLE("DBM0"), MagicLE("Echo"), PluginCategory::RoomFx, false, true },
 		// LFO
-		{ LFOPlugin::Create, "", "LFO", MagicLE("OMPT"), MagicLE("LFO "), VSTPluginLib::catGenerator, false, true },
+		{ LFOPlugin::Create, "", "LFO", MagicLE("OMPT"), MagicLE("LFO "), PluginCategory::Generator, false, true },
 		// SymMOD Echo
-		{ SymMODEcho::Create, "", "SymMOD Echo", MagicLE("SymM"), MagicLE("Echo"), VSTPluginLib::catRoomFx, false, true },
+		{ SymMODEcho::Create, "", "SymMOD Echo", MagicLE("SymM"), MagicLE("Echo"), PluginCategory::RoomFx, false, true },
 #ifdef MODPLUG_TRACKER
-		{ MidiInOut::Create, "", "MIDI Input Output", PLUGMAGIC('V','s','t','P'), PLUGMAGIC('M','M','I','D'), VSTPluginLib::catSynth, true, true },
+		{ MidiInOut::Create, "", "MIDI Input Output", PLUGMAGIC('V','s','t','P'), PLUGMAGIC('M','M','I','D'), PluginCategory::Synth, true, true },
 #endif // MODPLUG_TRACKER
 	};
 
 	pluginList.reserve(std::size(BuiltInPlugins));
 	for(const auto &plugin : BuiltInPlugins)
 	{
-		VSTPluginLib *plug = new (std::nothrow) VSTPluginLib(plugin.createProc, true, mpt::PathString::FromUTF8(plugin.filename), mpt::PathString::FromUTF8(plugin.name));
-		if(plug != nullptr)
-		{
-			pluginList.push_back(plug);
-			plug->pluginId1 = plugin.pluginId1;
-			plug->pluginId2 = plugin.pluginId2;
-			plug->category = plugin.category;
-			plug->isInstrument = plugin.isInstrument;
+		auto &plug = pluginList.emplace_back(std::make_unique<VSTPluginLib>(plugin.createProc, true, mpt::PathString::FromUTF8(plugin.filename), mpt::PathString::FromUTF8(plugin.name)));
+		plug->pluginId1 = plugin.pluginId1;
+		plug->pluginId2 = plugin.pluginId2;
+		plug->category = plugin.category;
+		plug->isInstrument = plugin.isInstrument;
 #ifdef MODPLUG_TRACKER
-			if(plugin.isOurs)
-				plug->vendor = _T("OpenMPT Project");
+		if(plugin.isOurs)
+			plug->vendor = _T("OpenMPT Project");
 #endif // MODPLUG_TRACKER
-		}
 	}
 
 #ifdef MODPLUG_TRACKER
@@ -341,9 +331,10 @@ CVstPluginManager::~CVstPluginManager()
 	{
 		while(plug->pPluginsList != nullptr)
 		{
-			plug->pPluginsList->Release();
+			IMixPlugin *pluginInstance = plug->pPluginsList;
+			plug->RemovePluginInstanceFromList(*pluginInstance);
+			pluginInstance->Release();
 		}
-		delete plug;
 	}
 #if defined(MPT_WITH_DMO)
 	if(MustUnInitilizeCOM)
@@ -357,7 +348,10 @@ CVstPluginManager::~CVstPluginManager()
 
 bool CVstPluginManager::IsValidPlugin(const VSTPluginLib *pLib) const
 {
-	return mpt::contains(pluginList, pLib);
+	return std::find_if(pluginList.begin(), pluginList.end(), [pLib](const std::unique_ptr<VSTPluginLib> &value)
+	{
+		return value.get() == pLib;
+	}) != pluginList.end();
 }
 
 
@@ -377,6 +371,7 @@ void CVstPluginManager::EnumerateDirectXDMOs()
 		"120CED89-3BF4-4173-A132-3CB406CF3231"_uuid, // ParamEq
 		"87FC0268-9A55-4360-95AA-004A1D9DE26C"_uuid, // WavesReverb
 		"F447B69E-1884-4A7E-8055-346F74D6EDB3"_uuid, // Resampler DMO (not usable)
+		"A8122FF4-9E52-4374-B3D9-B4063E77109D"_uuid, // XnaVisualizerDmo (not usable)
 	};
 
 	HKEY hkEnum;
@@ -402,26 +397,15 @@ void CVstPluginManager::EnumerateDirectXDMOs()
 						DWORD datatype = REG_SZ;
 						DWORD datasize = sizeof(name);
 
-						if(ERROR_SUCCESS == RegQueryValueEx(hksub, nullptr, 0, &datatype, (LPBYTE)name, &datasize))
+						if(ERROR_SUCCESS == RegQueryValueEx(hksub, nullptr, 0, &datatype, reinterpret_cast<LPBYTE>(name), &datasize))
 						{
-							VSTPluginLib *plug = new (std::nothrow) VSTPluginLib(DMOPlugin::Create, true, mpt::PathString::FromNative(mpt::GUIDToString(clsid)), mpt::PathString::FromNative(ParseMaybeNullTerminatedStringFromBufferWithSizeInBytes<mpt::winstring>(name, datasize)));
-							if(plug != nullptr)
-							{
-								try
-								{
-									pluginList.push_back(plug);
-									plug->pluginId1 = kDmoMagic;
-									plug->pluginId2 = clsid.Data1;
-									plug->category = VSTPluginLib::catDMO;
-								} catch(mpt::out_of_memory e)
-								{
-									mpt::delete_out_of_memory(e);
-									delete plug;
-								}
+							auto &plug = pluginList.emplace_back(std::make_unique<VSTPluginLib>(DMOPlugin::Create, true, mpt::PathString::FromNative(mpt::GUIDToString(clsid)), mpt::PathString::FromNative(ParseMaybeNullTerminatedStringFromBufferWithSizeInBytes<mpt::winstring>(name, datasize))));
+							plug->pluginId1 = kDmoMagic;
+							plug->pluginId2 = clsid.Data1;
+							plug->category = PluginCategory::DMO;
 #ifdef DMO_LOG
-								MPT_LOG_GLOBAL(LogDebug, "DMO", MPT_UFORMAT("Found \"{}\" clsid={}\n")(plug->libraryName, plug->dllPath));
+							MPT_LOG_GLOBAL(LogDebug, "DMO", MPT_UFORMAT("Found \"{}\" clsid={}\n")(plug->libraryName, plug->dllPath));
 #endif
-							}
 						}
 						RegCloseKey(hksub);
 					}
@@ -435,126 +419,103 @@ void CVstPluginManager::EnumerateDirectXDMOs()
 }
 
 
-// Extract instrument and category information from plugin.
 #ifdef MPT_WITH_VST
-static void GetPluginInformation(bool maskCrashes, Vst::AEffect *effect, VSTPluginLib &library)
+// Convert CVstPlugin::LoadResult into a collection of VSTPluginLibs.
+static std::vector<VSTPluginLib> GetPluginInformation(VSTPluginLib plug, const CVstPlugin::LoadResult &loadResult)
 {
-	unsigned long exception = 0;
-	library.category = static_cast<VSTPluginLib::PluginCategory>(CVstPlugin::DispatchSEH(maskCrashes, effect, Vst::effGetPlugCategory, 0, 0, nullptr, 0, exception));
-	library.isInstrument = ((effect->flags & Vst::effFlagsIsSynth) || !effect->numInputs);
+	plug.pluginId1 = loadResult.magic;
+	plug.pluginId2 = loadResult.uniqueID;
+	plug.isInstrument = CVstPlugin::IsInstrument(*loadResult.effect);
 
-	if(library.isInstrument)
+	std::vector<VSTPluginLib> containedPlugins;
+	if(loadResult.shellPlugins.empty())
 	{
-		library.category = VSTPluginLib::catSynth;
-	} else if(library.category >= VSTPluginLib::numCategories)
-	{
-		library.category = VSTPluginLib::catUnknown;
-	}
-
 #ifdef MODPLUG_TRACKER
-	std::vector<char> s(256, 0);
-	CVstPlugin::DispatchSEH(maskCrashes, effect, Vst::effGetVendorString, 0, 0, s.data(), 0, exception);
-	library.vendor = mpt::ToCString(mpt::Charset::Locale, s.data());
-#endif // MODPLUG_TRACKER
-}
-#endif // MPT_WITH_VST
-
-
-#ifdef MPT_WITH_VST
-static bool TryLoadPlugin(bool maskCrashes, VSTPluginLib *plug, HINSTANCE hLib, unsigned long &exception)
-{
-	Vst::AEffect *pEffect = CVstPlugin::LoadPlugin(maskCrashes, *plug, hLib, CVstPlugin::BridgeMode::DetectRequiredBridgeMode);
-	if(!pEffect || pEffect->magic != Vst::kEffectMagic || !pEffect->dispatcher)
+		plug.WriteToCache();
+#endif  // MODPLUG_TRACKER
+		containedPlugins.push_back(std::move(plug));
+	} else
 	{
-		return false;
+		for(auto &shellPlug : loadResult.shellPlugins)
+		{
+			plug.shellPluginID = shellPlug.shellPluginID;
+			plug.libraryName = mpt::PathString::FromLocale(shellPlug.name);
+#ifdef MODPLUG_TRACKER
+			plug.WriteToCache();
+#endif  // MODPLUG_TRACKER
+			containedPlugins.push_back(plug);
+		}
 	}
 
-	CVstPlugin::DispatchSEH(maskCrashes, pEffect, Vst::effOpen, 0, 0, 0, 0, exception);
-
-	plug->pluginId1 = pEffect->magic;
-	plug->pluginId2 = pEffect->uniqueID;
-
-	GetPluginInformation(maskCrashes, pEffect, *plug);
-
-#ifdef VST_LOG
-	intptr_t nver = CVstPlugin::DispatchSEH(maskCrashes, pEffect, Vst::effGetVstVersion, 0,0, nullptr, 0, exception);
-	if (!nver) nver = pEffect->version;
-	MPT_LOG_GLOBAL(LogDebug, "VST", MPT_UFORMAT("{}: v{}.0, {} in, {} out, {} programs, {} params, flags=0x{} realQ={} offQ={}")(
-		plug->libraryName, nver,
-		pEffect->numInputs, pEffect->numOutputs,
-		mpt::ufmt::dec0<2>(pEffect->numPrograms), mpt::ufmt::dec0<2>(pEffect->numParams),
-		mpt::ufmt::HEX0<4>(static_cast<int32>(pEffect->flags)), pEffect->realQualities, pEffect->offQualities));
-#endif // VST_LOG
-
-	CVstPlugin::DispatchSEH(maskCrashes, pEffect, Vst::effClose, 0, 0, 0, 0, exception);
-
-	return true;
+	return containedPlugins;
 }
-#endif // !NO_NVST
+#endif // !NO_VST
 
 
 #ifdef MODPLUG_TRACKER
 // Add a plugin to the list of known plugins.
-VSTPluginLib *CVstPluginManager::AddPlugin(const mpt::PathString &dllPath, bool maskCrashes, const mpt::ustring &tags, bool fromCache, bool *fileFound)
+std::vector<VSTPluginLib *> CVstPluginManager::AddPlugin(const mpt::PathString &dllPath, bool maskCrashes, bool fromCache, bool *fileFound, uint32 shellPluginID)
 {
-	const mpt::PathString fileName = dllPath.GetFileName();
+	const mpt::PathString fileName = dllPath.GetFilenameBase();
 
-	// Check if this is already a known plugin.
+	// Check if this is already a known plugin
 	for(const auto &dupePlug : pluginList)
 	{
-		if(!dllPath.CompareNoCase(dllPath, dupePlug->dllPath)) return dupePlug;
+		if(shellPluginID == dupePlug->shellPluginID && !mpt::PathCompareNoCase(dllPath, dupePlug->dllPath))
+			return {dupePlug.get()};
 	}
 
 	if(fileFound != nullptr)
 	{
-		*fileFound = dllPath.IsFile();
+		*fileFound = mpt::native_fs{}.is_file(dllPath);
 	}
 
 	// Look if the plugin info is stored in the PluginCache
 	if(fromCache)
 	{
-		SettingsContainer & cacheFile = theApp.GetPluginCache();
+		mpt::ustring shellStr;
+		if(shellPluginID)
+			shellStr = UL_("|") + mpt::ufmt::HEX0<8>(shellPluginID);
+
+		SettingsContainer &cacheFile = theApp.GetPluginCache();
 		// First try finding the full path
-		mpt::ustring IDs = cacheFile.Read<mpt::ustring>(cacheSection, dllPath.ToUnicode(), U_(""));
+		mpt::ustring IDs = cacheFile.Read<mpt::ustring>(cacheSection, dllPath.ToUnicode() + shellStr, U_(""));
 		if(IDs.length() < 16)
 		{
 			// If that didn't work out, find relative path
 			mpt::PathString relPath = theApp.PathAbsoluteToInstallRelative(dllPath);
-			IDs = cacheFile.Read<mpt::ustring>(cacheSection, relPath.ToUnicode(), U_(""));
+			IDs = cacheFile.Read<mpt::ustring>(cacheSection, relPath.ToUnicode() + shellStr, U_(""));
 		}
 
 		if(IDs.length() >= 16)
 		{
-			VSTPluginLib *plug = new (std::nothrow) VSTPluginLib(nullptr, false, dllPath, fileName, tags);
-			if(plug == nullptr)
-			{
-				return nullptr;
-			}
-			pluginList.push_back(plug);
+			auto &plug = pluginList.emplace_back(std::make_unique<VSTPluginLib>(nullptr, false, dllPath, fileName));
 
 			// Extract plugin IDs
-			for (int i = 0; i < 16; i++)
+			uint32 id1 = 0, id2 = 0;
+			for(int i = 0; i < 16; i++)
 			{
-				int32 n = IDs[i] - '0';
-				if (n > 9) n = IDs[i] + 10 - 'A';
-				n &= 0x0f;
+				uint32 n = IDs[i] - '0';
+				if(n > 9)
+					n = IDs[i] + 10 - 'A';
+				n &= 0x0F;
 				if (i < 8)
-				{
-					plug->pluginId1 = (plug->pluginId1 << 4) | n;
-				} else
-				{
-					plug->pluginId2 = (plug->pluginId2 << 4) | n;
-				}
+					id1 = (id1 << 4) | n;
+				else
+					id2 = (id2 << 4) | n;
 			}
+			plug->pluginId1 = id1;
+			plug->pluginId2 = id2;
 
 			const mpt::ustring flagKey = IDs + U_(".Flags");
 			plug->DecodeCacheFlags(cacheFile.Read<int32>(cacheSection, flagKey, 0));
 			plug->vendor = cacheFile.Read<CString>(cacheSection, IDs + U_(".Vendor"), CString());
+			plug->shellPluginID = shellPluginID;
 
 #ifdef VST_LOG
 			MPT_LOG_GLOBAL(LogDebug, "VST", MPT_UFORMAT("Plugin \"{}\" found in PluginCache")(plug->libraryName));
 #endif // VST_LOG
-			return plug;
+			return {plug.get()};
 		} else
 		{
 #ifdef VST_LOG
@@ -566,51 +527,54 @@ VSTPluginLib *CVstPluginManager::AddPlugin(const mpt::PathString &dllPath, bool
 	// If this key contains a file name on program launch, a plugin previously crashed OpenMPT.
 	theApp.GetSettings().Write<mpt::PathString>(U_("VST Plugins"), U_("FailedPlugin"), dllPath, SettingWriteThrough);
 
-	bool validPlug = false;
-
-	VSTPluginLib *plug = new (std::nothrow) VSTPluginLib(nullptr, false, dllPath, fileName, tags);
-	if(plug == nullptr)
-	{
-		return nullptr;
-	}
+	std::vector<VSTPluginLib *> foundPlugins;
 
 #ifdef MPT_WITH_VST
 	unsigned long exception = 0;
 	// Always scan plugins in a separate process
-	HINSTANCE hLib = NULL;
 	{
-#ifdef MODPLUG_TRACKER
-		ExceptionHandler::Context ectx{ MPT_UFORMAT("VST Plugin: {}")(plug->dllPath.ToUnicode()) };
+		ExceptionHandler::Context ectx{ MPT_UFORMAT("VST Plugin: {}")(dllPath.ToUnicode()) };
 		ExceptionHandler::ContextSetter ectxguard{&ectx};
-#endif // MODPLUG_TRACKER
 
-		validPlug = TryLoadPlugin(maskCrashes, plug, hLib, exception);
-	}
-	if(hLib)
-	{
-		FreeLibrary(hLib);
+		VSTPluginLib plug{nullptr, false, dllPath, fileName};
+		auto loadResult = CVstPlugin::LoadPlugin(maskCrashes, plug, CVstPlugin::BridgeMode::DetectRequiredBridgeMode, exception);
+		Vst::AEffect *pEffect = loadResult.effect;
+		if(pEffect)
+		{
+			foundPlugins = AddPluginsToList(GetPluginInformation(std::move(plug), loadResult),
+				[&](VSTPluginLib &library, bool updateExisting)
+				{
+					if(updateExisting)
+						return;
+					if(library.shellPluginID)
+					{
+						if(!CVstPlugin::SelectShellPlugin(maskCrashes, loadResult, library))
+							return;
+					}
+					CVstPlugin::GetPluginMetadata(maskCrashes, loadResult, library);
+				});
+			CVstPlugin::DispatchSEH(maskCrashes, *pEffect, Vst::effClose, 0, 0, 0, 0, exception);
+		}
+		if(loadResult.library)
+		{
+			FreeLibrary(loadResult.library);
+		}
 	}
 	if(exception != 0)
 	{
-		CVstPluginManager::ReportPlugException(MPT_UFORMAT("Exception {} while trying to load plugin \"{}\"!\n")(mpt::ufmt::HEX0<8>(exception), plug->libraryName));
+		CVstPluginManager::ReportPlugException(MPT_UFORMAT("Exception {} while trying to load plugin \"{}\"!\n")(mpt::ufmt::HEX0<8>(exception), fileName));
 	}
 
+#else // !MPT_WITH_VST
+
+	MPT_UNUSED(maskCrashes);
+
 #endif // MPT_WITH_VST
 
 	// Now it should be safe to assume that this plugin loaded properly. :)
 	theApp.GetSettings().Remove(U_("VST Plugins"), U_("FailedPlugin"));
 
-	// If OK, write the information in PluginCache
-	if(validPlug)
-	{
-		pluginList.push_back(plug);
-		plug->WriteToCache();
-	} else
-	{
-		delete plug;
-	}
-
-	return (validPlug ? plug : nullptr);
+	return foundPlugins;
 }
 
 
@@ -619,7 +583,7 @@ bool CVstPluginManager::RemovePlugin(VSTPluginLib *pFactory)
 {
 	for(const_iterator p = begin(); p != end(); p++)
 	{
-		VSTPluginLib *plug = *p;
+		VSTPluginLib *plug = p->get();
 		if(plug == pFactory)
 		{
 			// Kill all instances of this plugin
@@ -627,10 +591,11 @@ bool CVstPluginManager::RemovePlugin(VSTPluginLib *pFactory)
 
 			while(plug->pPluginsList != nullptr)
 			{
-				plug->pPluginsList->Release();
+				IMixPlugin *pluginInstance = plug->pPluginsList;
+				plug->RemovePluginInstanceFromList(*pluginInstance);
+				pluginInstance->Release();
 			}
 			pluginList.erase(p);
-			delete plug;
 			return true;
 		}
 	}
@@ -653,8 +618,8 @@ bool CVstPluginManager::CreateMixPlugin(SNDMIXPLUGIN &mixPlugin, CSoundFile &snd
 		kMatchNameAndId,
 	};
 
-	PlugMatchQuality match = kNoMatch;	// "Match quality" of found plugin. Higher value = better match.
-#if MPT_OS_WINDOWS && !MPT_OS_WINDOWS_WINRT
+	PlugMatchQuality match = kNoMatch;  // "Match quality" of found plugin. Higher value = better match.
+#if defined(MODPLUG_TRACKER) && MPT_OS_WINDOWS && !MPT_OS_WINDOWS_WINRT
 	const mpt::PathString libraryName = mpt::PathString::FromUnicode(mixPlugin.GetLibraryName());
 #else
 	const std::string libraryName = mpt::ToCharset(mpt::Charset::UTF8, mixPlugin.GetLibraryName());
@@ -662,16 +627,17 @@ bool CVstPluginManager::CreateMixPlugin(SNDMIXPLUGIN &mixPlugin, CSoundFile &snd
 	for(const auto &plug : pluginList)
 	{
 		const bool matchID = (plug->pluginId1 == mixPlugin.Info.dwPluginId1)
-			&& (plug->pluginId2 == mixPlugin.Info.dwPluginId2);
-#if MPT_OS_WINDOWS && !MPT_OS_WINDOWS_WINRT
-		const bool matchName = !mpt::PathString::CompareNoCase(plug->libraryName, libraryName);
+			&& (plug->pluginId2 == mixPlugin.Info.dwPluginId2)
+			&& (plug->shellPluginID == mixPlugin.Info.shellPluginID);
+#if defined(MODPLUG_TRACKER) && MPT_OS_WINDOWS && !MPT_OS_WINDOWS_WINRT
+		const bool matchName = !mpt::PathCompareNoCase(plug->libraryName, libraryName);
 #else
 		const bool matchName = !mpt::CompareNoCaseAscii(plug->libraryName.ToUTF8(), libraryName);
 #endif
 
 		if(matchID && matchName)
 		{
-			pFound = plug;
+			pFound = plug.get();
 #ifdef MPT_WITH_VST
 			if(plug->IsNative(false))
 			{
@@ -682,100 +648,152 @@ bool CVstPluginManager::CreateMixPlugin(SNDMIXPLUGIN &mixPlugin, CSoundFile &snd
 			match = kMatchNameAndId;
 		} else if(matchID && match < kMatchId)
 		{
-			pFound = plug;
+			pFound = plug.get();
 			match = kMatchId;
 		} else if(matchName && match < kMatchName)
 		{
-			pFound = plug;
+			pFound = plug.get();
 			match = kMatchName;
 		}
 	}
 
-	if(pFound != nullptr && pFound->Create != nullptr)
+	if(!pFound)
+		return false;
+
+	IMixPlugin *plugin = nullptr;
+	if(pFound->Create != nullptr)
 	{
-		IMixPlugin *plugin = pFound->Create(*pFound, sndFile, &mixPlugin);
-		return plugin != nullptr;
+		plugin = pFound->Create(*pFound, sndFile, mixPlugin);
 	}
+#ifdef MPT_WITH_VST
+	// Note: we don't check if dwPluginId1 matches Vst::kEffectMagic here, even if it should.
+	// I have an old file I made with OpenMPT 1.17 where the primary plugin ID has an unexpected value.
+	// No idea how that could happen, apart from some plugin.cache corruption (back then, the IDs were not re-checked
+	// after instantiating a plugin and the cached plugin ID was blindly written to the module file)
+	else
+	{
+		bool maskCrashes = TrackerSettings::Instance().BrokenPluginsWorkaroundVSTMaskAllCrashes;
+
+		unsigned long exception = 0;
+		auto loadResult = CVstPlugin::LoadPlugin(maskCrashes, *pFound, TrackerSettings::Instance().bridgeAllPlugins ? CVstPlugin::BridgeMode::ForceBridgeWithFallback : CVstPlugin::BridgeMode::Automatic, exception);
+		Vst::AEffect *pEffect = loadResult.effect;
+
+		if(pEffect != nullptr)
+		{
+			// If filename matched during load but plugin ID didn't (or vice versa), make sure it's updated.
+			pFound->pluginId1 = loadResult.magic;
+			pFound->pluginId2 = loadResult.uniqueID;
+
+			plugin = new (std::nothrow) CVstPlugin(maskCrashes, loadResult.library, *pFound, mixPlugin, *pEffect, sndFile);
 
 #ifdef MODPLUG_TRACKER
-	bool maskCrashes = TrackerSettings::Instance().BrokenPluginsWorkaroundVSTMaskAllCrashes;
-
-	if(!pFound && (mixPlugin.GetLibraryName() != U_("")))
-	{
-		// Try finding the plugin DLL in the plugin directory or plugin cache instead.
-		mpt::PathString fullPath = TrackerSettings::Instance().PathPlugins.GetDefaultDir();
-		if(fullPath.empty())
-		{
-			fullPath = theApp.GetInstallPath() + P_("Plugins\\");
-		}
-		fullPath += mpt::PathString::FromUnicode(mixPlugin.GetLibraryName()) + P_(".dll");
-
-		pFound = AddPlugin(fullPath, maskCrashes);
-		if(!pFound)
-		{
-			// Try plugin cache (search for library name)
-			SettingsContainer &cacheFile = theApp.GetPluginCache();
-			mpt::ustring IDs = cacheFile.Read<mpt::ustring>(cacheSection, mixPlugin.GetLibraryName(), U_(""));
-			if(IDs.length() >= 16)
-			{
-				fullPath = cacheFile.Read<mpt::PathString>(cacheSection, IDs, P_(""));
-				if(!fullPath.empty())
+			AddPluginsToList(GetPluginInformation(*pFound, loadResult),
+				[&](VSTPluginLib &library, bool updateExisting)
 				{
-					fullPath = theApp.PathInstallRelativeToAbsolute(fullPath);
-					if(fullPath.IsFile())
-					{
-						pFound = AddPlugin(fullPath, maskCrashes);
-					}
-				}
-			}
-		}
-	}
-
-#ifdef MPT_WITH_VST
-	if(pFound && mixPlugin.Info.dwPluginId1 == Vst::kEffectMagic)
-	{
-		Vst::AEffect *pEffect = nullptr;
-		HINSTANCE hLibrary = nullptr;
-		bool validPlugin = false;
-
-		pEffect = CVstPlugin::LoadPlugin(maskCrashes, *pFound, hLibrary, TrackerSettings::Instance().bridgeAllPlugins ? CVstPlugin::BridgeMode::ForceBridgeWithFallback : CVstPlugin::BridgeMode::Automatic);
-
-		if(pEffect != nullptr && pEffect->dispatcher != nullptr && pEffect->magic == Vst::kEffectMagic)
-		{
-			validPlugin = true;
-
-			GetPluginInformation(maskCrashes, pEffect, *pFound);
-
-			// Update cached information
-			pFound->WriteToCache();
-
-			CVstPlugin *pVstPlug = new (std::nothrow) CVstPlugin(maskCrashes, hLibrary, *pFound, mixPlugin, *pEffect, sndFile);
-			if(pVstPlug == nullptr)
-			{
-				validPlugin = false;
-			}
+					if(&library == pFound && updateExisting)
+						CVstPlugin::GetPluginMetadata(maskCrashes, loadResult, library);
+				});
+#endif
 		}
 
-		if(!validPlugin)
+		if(!plugin && loadResult.library)
 		{
-			FreeLibrary(hLibrary);
+			FreeLibrary(loadResult.library);
 			CVstPluginManager::ReportPlugException(MPT_UFORMAT("Unable to create plugin \"{}\"!\n")(pFound->libraryName));
 		}
-		return validPlugin;
-	} else
-	{
-		// "plug not found" notification code MOVED to CSoundFile::Create
-#ifdef VST_LOG
-		MPT_LOG_GLOBAL(LogDebug, "VST", U_("Unknown plugin"));
-#endif
 	}
 #endif // MPT_WITH_VST
 
-#endif // MODPLUG_TRACKER
-	return false;
+#ifdef MODPLUG_TRACKER
+	CriticalSection cs;
+#endif
+	if(plugin)
+		pFound->InsertPluginInstanceIntoList(*plugin);
+	mixPlugin.pMixPlugin = plugin;
+	// If filename matched during load but plugin ID didn't (or vice versa), make sure it's updated.
+	mixPlugin.Info.dwPluginId1 = pFound->pluginId1;
+	mixPlugin.Info.dwPluginId2 = pFound->pluginId2;
+	mixPlugin.Info.szLibraryName = pFound->libraryName.ToUTF8();
+
+	return plugin != nullptr;
 }
 
 
+#ifdef MPT_WITH_VST
+std::vector<VSTPluginLib *> CVstPluginManager::AddPluginsToList(std::vector<VSTPluginLib> containedPlugins, std::function<void(VSTPluginLib &, bool)> updateFunc)
+{
+	std::vector<VSTPluginLib *> newPlugins;
+	if(containedPlugins.empty())
+		return newPlugins;
+
+	// Find existing shell plugins belonging to the same file first, so that we don't have to iterate through the whole plugin list again and again
+	std::map<uint32, size_t> existingCandidates;
+	for(size_t i = 0; i < pluginList.size(); i++)
+	{
+		const auto &plug = pluginList[i];
+		if(plug->pluginId1 == containedPlugins.front().pluginId1 && plug->pluginId2 == containedPlugins.front().pluginId2)
+		{
+			if(!mpt::PathCompareNoCase(plug->dllPath, containedPlugins.front().dllPath))
+				existingCandidates[plug->shellPluginID] = i;
+		}
+	}
+
+	// Add found plugins to list or update them if they already exist
+	std::set<uint32> containedIDs;
+	VSTPluginLib *first = nullptr;
+	for(auto &containedPlug : containedPlugins)
+	{
+		containedIDs.insert(containedPlug.shellPluginID);
+		VSTPluginLib *found = nullptr;
+		if(auto it = existingCandidates.find(containedPlug.shellPluginID); it != existingCandidates.end())
+		{
+			auto &plug = pluginList[it->second];
+			MPT_ASSERT(plug->pluginId1 == containedPlug.pluginId1 && plug->pluginId2 == containedPlug.pluginId2);
+			if(plug->shellPluginID == containedPlug.shellPluginID)
+				found = plug.get();
+		}
+		const bool updateExisting = found != nullptr;
+		if(updateExisting)
+		{
+			found->libraryName = containedPlug.libraryName;
+		} else
+		{
+			auto &plug = pluginList.emplace_back(std::make_unique<VSTPluginLib>(std::move(containedPlug)));
+			found = plug.get();
+			newPlugins.push_back(found);
+		}
+		updateFunc(*found, updateExisting);
+
+		if(!first)
+			first = found;
+			
+#ifdef MODPLUG_TRACKER
+		found->WriteToCache();
+#endif // MODPLUG_TRACKER
+	}
+
+	// Are there any shell plugins in our list that are no longer part of the shell plugin?
+	if(containedPlugins.size() > 1)
+	{
+		size_t deleted = 0;
+		for(const auto &[id, i] : existingCandidates)
+		{
+			if(!mpt::contains(containedIDs, id) && !pluginList[i - deleted]->pPluginsList)
+			{
+				MPT_ASSERT(pluginList[i - deleted]->shellPluginID == id);
+				pluginList.erase(pluginList.begin() + i - deleted);
+				deleted++;
+			}
+		}
+	}
+
+	if(newPlugins.empty() && first)
+		newPlugins.push_back(first);
+	return newPlugins;
+}
+#endif // MPT_WITH_VST
+
+
 #ifdef MODPLUG_TRACKER
 void CVstPluginManager::OnIdle()
 {
