@@ -27,13 +27,13 @@ namespace DMO
 {
 
 
-IMixPlugin* Compressor::Create(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN *mixStruct)
+IMixPlugin* Compressor::Create(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN &mixStruct)
 {
 	return new (std::nothrow) Compressor(factory, sndFile, mixStruct);
 }
 
 
-Compressor::Compressor(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN *mixStruct)
+Compressor::Compressor(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN &mixStruct)
 	: IMixPlugin(factory, sndFile, mixStruct)
 {
 	m_param[kCompGain] = 0.5f;
@@ -44,7 +44,6 @@ Compressor::Compressor(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN
 	m_param[kCompPredelay] = 1.0f;
 
 	m_mixBuffer.Initialize(2, 2);
-	InsertIntoFactoryList();
 }
 
 
@@ -89,11 +88,11 @@ void Compressor::Process(float *pOutL, float *pOutR, uint32 numFrames)
 		}
 		compGainPow >>= (31 - compGainInt);
 		
-		int32 readOffset = m_predelay + m_bufPos * 4096 + m_bufSize - 1;
+		int32 readOffset = m_predelay + m_bufSize - 1;
 		readOffset /= 4096;
-		readOffset %= m_bufSize;
-		
-		float outGain = (compGainPow * (1.0f / 2147483648.0f)) * m_gain;
+		readOffset = (readOffset + m_bufPos) % m_bufSize;
+
+		float outGain = (static_cast<float>(compGainPow) * (1.0f / 2147483648.0f)) * m_gain;
 		*(out[0])++ = m_buffer[readOffset * 2] * outGain;
 		*(out[1])++ = m_buffer[readOffset * 2 + 1] * outGain;
 		
@@ -115,7 +114,7 @@ PlugParamValue Compressor::GetParameter(PlugParamIndex index)
 }
 
 
-void Compressor::SetParameter(PlugParamIndex index, PlugParamValue value)
+void Compressor::SetParameter(PlugParamIndex index, PlugParamValue value, PlayState *, CHANNELINDEX)
 {
 	if(index < kCompNumParameters)
 	{
@@ -178,6 +177,8 @@ CString Compressor::GetParamLabel(PlugParamIndex param)
 	case kCompRelease:
 	case kCompPredelay:
 		return _T("ms");
+	case kCompRatio:
+		return _T(": 1");
 	}
 	return CString();
 }
@@ -217,7 +218,7 @@ CString Compressor::GetParamDisplay(PlugParamIndex param)
 
 void Compressor::RecalculateCompressorParams()
 {
-	const float sampleRate = m_SndFile.GetSampleRate() / 1000.0f;
+	const float sampleRate = static_cast<float>(m_SndFile.GetSampleRate()) / 1000.0f;
 	m_gain = std::pow(10.0f, GainInDecibel() / 20.0f);
 	m_attack = std::pow(10.0f, -1.0f / (AttackTime() * sampleRate));
 	m_release = std::pow(10.0f, -1.0f / (ReleaseTime() * sampleRate));
