@@ -11,9 +11,9 @@
 #include "stdafx.h"
 
 #ifndef NO_PLUGINS
-#include "../../Sndfile.h"
 #include "Gargle.h"
-#endif // !NO_PLUGINS
+#include "../../Sndfile.h"
+#endif  // !NO_PLUGINS
 
 OPENMPT_NAMESPACE_BEGIN
 
@@ -22,20 +22,19 @@ OPENMPT_NAMESPACE_BEGIN
 namespace DMO
 {
 
-IMixPlugin* Gargle::Create(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN *mixStruct)
+IMixPlugin* Gargle::Create(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN &mixStruct)
 {
 	return new (std::nothrow) Gargle(factory, sndFile, mixStruct);
 }
 
 
-Gargle::Gargle(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN *mixStruct)
+Gargle::Gargle(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN &mixStruct)
 	: IMixPlugin(factory, sndFile, mixStruct)
 {
 	m_param[kGargleRate] = 0.02f;
 	m_param[kGargleWaveShape] = 0.0f;
 
 	m_mixBuffer.Initialize(2, 2);
-	InsertIntoFactoryList();
 }
 
 
@@ -57,11 +56,11 @@ void Gargle::Process(float *pOutL, float *pOutR, uint32 numFrames)
 			if(triangle)
 			{
 				const uint32 stop = m_counter + remain;
-				const float factor = 1.0f / m_periodHalf;
+				const float factor = 1.0f / static_cast<float>(m_periodHalf);
 				for(uint32 i = m_counter; i < stop; i++)
 				{
-					*outL++ = *inL++ * i * factor;
-					*outR++ = *inR++ * i * factor;
+					*outL++ = *inL++ * static_cast<float>(i) * factor;
+					*outR++ = *inR++ * static_cast<float>(i) * factor;
 				}
 			} else
 			{
@@ -80,11 +79,11 @@ void Gargle::Process(float *pOutL, float *pOutR, uint32 numFrames)
 			if(triangle)
 			{
 				const uint32 stop = m_period - m_counter - remain;
-				const float factor = 1.0f / m_periodHalf;
+				const float factor = 1.0f / static_cast<float>(m_periodHalf);
 				for(uint32 i = m_period - m_counter; i > stop; i--)
 				{
-					*outL++ = *inL++ * i * factor;
-					*outR++ = *inR++ * i * factor;
+					*outL++ = *inL++ * static_cast<float>(i) * factor;
+					*outR++ = *inR++ * static_cast<float>(i) * factor;
 				}
 			} else
 			{
@@ -117,7 +116,7 @@ PlugParamValue Gargle::GetParameter(PlugParamIndex index)
 }
 
 
-void Gargle::SetParameter(PlugParamIndex index, PlugParamValue value)
+void Gargle::SetParameter(PlugParamIndex index, PlugParamValue value, PlayState *, CHANNELINDEX)
 {
 	if(index < kGargleNumParameters)
 	{
