@@ -6,7 +6,7 @@
 
 #include "openmpt/all/BuildSettings.hpp"
 
-#include "mpt/base/floatingpoint.hpp"
+#include "mpt/base/float.hpp"
 #include "mpt/base/macros.hpp"
 #include "mpt/base/memory.hpp"
 #include "openmpt/base/Endian.hpp"
@@ -187,7 +187,7 @@ struct DecodeInt16
 	static constexpr std::size_t input_inc = 2;
 	MPT_FORCEINLINE output_t operator()(const input_t *inBuf)
 	{
-		return (mpt::byte_cast<uint8>(inBuf[loByteIndex]) | (mpt::byte_cast<uint8>(inBuf[hiByteIndex]) << 8)) - offset;
+		return static_cast<int16>((mpt::byte_cast<uint8>(inBuf[loByteIndex]) | (mpt::byte_cast<uint8>(inBuf[hiByteIndex]) << 8)) - offset);
 	}
 };
 
@@ -204,7 +204,7 @@ struct DecodeInt16Delta
 	}
 	MPT_FORCEINLINE output_t operator()(const input_t *inBuf)
 	{
-		delta += mpt::byte_cast<uint8>(inBuf[loByteIndex]) | (mpt::byte_cast<uint8>(inBuf[hiByteIndex]) << 8);
+		delta += static_cast<uint16>(static_cast<uint16>(mpt::byte_cast<uint8>(inBuf[loByteIndex])) | static_cast<uint16>(mpt::byte_cast<uint8>(inBuf[hiByteIndex]) << 8));
 		return static_cast<int16>(delta);
 	}
 };
@@ -224,7 +224,7 @@ struct DecodeInt16Delta8
 		delta += mpt::byte_cast<uint8>(inBuf[0]);
 		int16 result = delta & 0xFF;
 		delta += mpt::byte_cast<uint8>(inBuf[1]);
-		result |= (delta << 8);
+		result |= static_cast<uint16>(delta << 8);
 		return result;
 	}
 };
@@ -237,7 +237,12 @@ struct DecodeInt24
 	static constexpr std::size_t input_inc = 3;
 	MPT_FORCEINLINE output_t operator()(const input_t *inBuf)
 	{
-		return ((mpt::byte_cast<uint8>(inBuf[loByteIndex]) << 8) | (mpt::byte_cast<uint8>(inBuf[midByteIndex]) << 16) | (mpt::byte_cast<uint8>(inBuf[hiByteIndex]) << 24)) - offset;
+		const uint32 tmp = (uint32(0)
+							| (static_cast<uint32>(mpt::byte_cast<uint8>(inBuf[loByteIndex])) << 8)
+							| (static_cast<uint32>(mpt::byte_cast<uint8>(inBuf[midByteIndex])) << 16)
+							| (static_cast<uint32>(mpt::byte_cast<uint8>(inBuf[hiByteIndex])) << 24))
+						 - offset;
+		return static_cast<int32>(tmp);
 	}
 };
 
@@ -249,7 +254,13 @@ struct DecodeInt32
 	static constexpr std::size_t input_inc = 4;
 	MPT_FORCEINLINE output_t operator()(const input_t *inBuf)
 	{
-		return (mpt::byte_cast<uint8>(inBuf[loLoByteIndex]) | (mpt::byte_cast<uint8>(inBuf[loHiByteIndex]) << 8) | (mpt::byte_cast<uint8>(inBuf[hiLoByteIndex]) << 16) | (mpt::byte_cast<uint8>(inBuf[hiHiByteIndex]) << 24)) - offset;
+		const uint32 tmp = (uint32(0)
+							| (static_cast<uint32>(mpt::byte_cast<uint8>(inBuf[loLoByteIndex])) << 0)
+							| (static_cast<uint32>(mpt::byte_cast<uint8>(inBuf[loHiByteIndex])) << 8)
+							| (static_cast<uint32>(mpt::byte_cast<uint8>(inBuf[hiLoByteIndex])) << 16)
+							| (static_cast<uint32>(mpt::byte_cast<uint8>(inBuf[hiHiByteIndex])) << 24))
+						 - offset;
+		return static_cast<int32>(tmp);
 	}
 };
 
@@ -261,16 +272,17 @@ struct DecodeInt64
 	static constexpr std::size_t input_inc = 8;
 	MPT_FORCEINLINE output_t operator()(const input_t *inBuf)
 	{
-		return (uint64(0)
-				| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b0])) << 0)
-				| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b1])) << 8)
-				| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b2])) << 16)
-				| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b3])) << 24)
-				| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b4])) << 32)
-				| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b5])) << 40)
-				| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b6])) << 48)
-				| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b7])) << 56))
-			 - offset;
+		const uint64 tmp = (uint64(0)
+							| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b0])) << 0)
+							| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b1])) << 8)
+							| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b2])) << 16)
+							| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b3])) << 24)
+							| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b4])) << 32)
+							| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b5])) << 40)
+							| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b6])) << 48)
+							| (static_cast<uint64>(mpt::byte_cast<uint8>(inBuf[b7])) << 56))
+						 - offset;
+		return static_cast<int64>(tmp);
 	}
 };
 
@@ -278,11 +290,11 @@ template <std::size_t loLoByteIndex, std::size_t loHiByteIndex, std::size_t hiLo
 struct DecodeFloat32
 {
 	using input_t = std::byte;
-	using output_t = float32;
+	using output_t = somefloat32;
 	static constexpr std::size_t input_inc = 4;
 	MPT_FORCEINLINE output_t operator()(const input_t *inBuf)
 	{
-		float32 val = IEEE754binary32LE(inBuf[loLoByteIndex], inBuf[loHiByteIndex], inBuf[hiLoByteIndex], inBuf[hiHiByteIndex]);
+		somefloat32 val = IEEE754binary32LE(inBuf[loLoByteIndex], inBuf[loHiByteIndex], inBuf[hiLoByteIndex], inBuf[hiHiByteIndex]);
 		val = mpt::sanitize_nan(val);
 		if(std::isinf(val))
 		{
@@ -302,12 +314,12 @@ template <std::size_t loLoByteIndex, std::size_t loHiByteIndex, std::size_t hiLo
 struct DecodeScaledFloat32
 {
 	using input_t = std::byte;
-	using output_t = float32;
+	using output_t = somefloat32;
 	static constexpr std::size_t input_inc = 4;
 	float factor;
 	MPT_FORCEINLINE output_t operator()(const input_t *inBuf)
 	{
-		float32 val = IEEE754binary32LE(inBuf[loLoByteIndex], inBuf[loHiByteIndex], inBuf[hiLoByteIndex], inBuf[hiHiByteIndex]);
+		somefloat32 val = IEEE754binary32LE(inBuf[loLoByteIndex], inBuf[loHiByteIndex], inBuf[hiLoByteIndex], inBuf[hiHiByteIndex]);
 		val = mpt::sanitize_nan(val);
 		if(std::isinf(val))
 		{
@@ -332,11 +344,11 @@ template <std::size_t b0, std::size_t b1, std::size_t b2, std::size_t b3, std::s
 struct DecodeFloat64
 {
 	using input_t = std::byte;
-	using output_t = float64;
+	using output_t = somefloat64;
 	static constexpr std::size_t input_inc = 8;
 	MPT_FORCEINLINE output_t operator()(const input_t *inBuf)
 	{
-		float64 val = IEEE754binary64LE(inBuf[b0], inBuf[b1], inBuf[b2], inBuf[b3], inBuf[b4], inBuf[b5], inBuf[b6], inBuf[b7]);
+		somefloat64 val = IEEE754binary64LE(inBuf[b0], inBuf[b1], inBuf[b2], inBuf[b3], inBuf[b4], inBuf[b5], inBuf[b6], inBuf[b7]);
 		val = mpt::sanitize_nan(val);
 		if(std::isinf(val))
 		{
